#!/usr/bin/env python

"""RNAalignment

Example::

	# STOCKHOLM 1.0

    AACY023581040                --CGUUGGACU------AAA--------AGUCGGAAGUAAGC-----AAU-C------GCUGAAGCAACGC---
	AJ630128                     AUCGUUCAUUCGCUAUUCGCA-AAUAGCGAACGCAA--AAG------CCG-A-------CUGAAGGAACGGGAC
	target                       --CGUUGACCCAG----GAAA-----CUGGGCGGAAGUAAGGCCCAUUGCACUCCGGGCCUGAAGCAACGCG--
	#=GC SS_cons                 ::(((((,<<<<<<<.._____..>>>>>>>,,,,,,,,<<<<...._____.....>>>>,,,,)))))::::
	x                            --xxxxxxxxx-----------------xxxxxxxx--xxx------------------xxxxxxxxxxx----
	#=GC RF                      AUCGUUCAuCucccc..uuuuu..ggggaGaCGGAAGUAGGca....auaaa.....ugCCGAAGGAACGCguu
    //

x line is used to pick resides to calculate RMSD.

x line could be renamed to EvoClust"""
from __future__ import print_function
import warnings
warnings.filterwarnings("ignore")

from Bio import AlignIO


class RNAalignment:
    """RNAalignemnt"""

    def __init__(self, fn):
        """Load the alignment in the Stockholm format using biopython"""
        self.alignment = AlignIO.read(open(fn), "stockholm")

    def get_range(self, seqid, offset=0, verbose=True):
        """Get a list of positions for selected residues based on the last line of the alignment!

        If seqis not found in the alignment, raise an exception, like ::

            Exception: Seq not found in the alignment: 'CP000879.1/21644622164546

        .. warning:: EvoClust lines has to be -1 in the alignemnt."""
        # evoclust line
        x = self.alignment.get_all_seqs()[-1].seq  # ---(((((((----xxxxx--

        x_range = []
        seq_found = False
        for record in self.alignment:
            if record.id == seqid.strip():
                seq_found = True
                spos = 0
                for xi, si in zip(x, record.seq):
                    if si != '-':
                        spos += 1
                    if xi != '-':
                        # print xi, si,
                        # print si, spos
                        x_range.append(spos + offset)
        # if verbose: print '  # selected residues:', len(x_range)
        if not seq_found:
            raise Exception('Seq not found in the alignment: %s' % seqid)
        if not x_range:
            raise Exception('Seq not found or wrong x line')
        return x_range


if __name__ == '__main__':
    ra = RNAalignment('test_data/rp13finalX.stk')
    print(len(ra.get_range('rp13target', offset=0)))
    print(len(ra.get_range('cp0016', offset=0)))
    print(len(ra.get_range('NZ_ABBD01000528', offset=0)))

    print(ra.get_range('rp13target', offset=0))
    print(ra.get_range('cp0016', offset=0))
    print(ra.get_range('NZ_ABBD01000528', offset=0))



#!/usr/bin/env python

"""Calculate RMSD between structures based on a given alignment and xxxx lines.

When RNA models are loaded, models ending with 'template.pdb' are ignore.

c1_tha_96cdea07- tha in mapping
"""
from __future__ import print_function
import pandas as pd
pd.set_option('display.width', 1000)
import matplotlib.pyplot as plt
plt.style.use('ggplot')
import argparse

import sys
import math
import glob
import re
import os
import shutil

import Bio.PDB.PDBParser
import Bio.PDB.Superimposer
from Bio.PDB.PDBIO import Select
from Bio.PDB import PDBIO, Superimposer

from RNAalignment import RNAalignment
from RNAmodel import RNAmodel
import csv

debug = False


def get_rna_models_from_dir(directory, residues, save, output_dir):
    models = []
    if not os.path.exists(directory):
        raise Exception('Dir does not exist! ', directory)
    files = glob.glob(directory + "/*.pdb")
    files_sorted = sort_nicely(files)
    for f in files_sorted:
        # ignore files that can be found in your folder
        # be careful with this --magnus
        if f.endswith('template.pdb'):
            continue
        if 'clust01X' in f:
            continue
        if 'clust02X' in f:
            continue
        if 'clust03X' in f:
            continue
        models.append(RNAmodel(f, residues, save, output_dir))
    return models


def sort_nicely(l):
    """ Sort the given list in the way that humans expect.

    http://blog.codinghorror.com/sorting-for-humans-natural-sort-order/
    """
    def convert(text): return int(text) if text.isdigit() else text

    def alphanum_key(key): return [convert(c) for c in re.split('([0-9]+)', key)]
    l.sort(key=alphanum_key)
    return l


def parse_num_list(s):
    """ http://stackoverflow.com/questions/6512280/accept-a-range-of-numbers-in-the-form-of-0-5-using-pythons-argparse """
    m = re.match(r'(\d+)(?:-(\d+))?$', str(s))
    # ^ (or use .split('-'). anyway you like.)
    if not m:
        return s
    start = m.group(1)
    end = m.group(2) or start
    return list(range(int(start, 10), int(end, 10) + 1))

# def pair:
#    def __init__(self, f1, f2):
#        self.f1
#        self.f2
#    def calc_distance():
#        pass


def get_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument('-a', "--rna_alignment_fn",
                        help="rna alignemnt with the extra guidance line, e.g. test_data/rp14sub.stk", required=True)
    parser.add_argument('-t', "--target", help="the native structure file", required=True)
    parser.add_argument('-o', "--output_fn", help="output csv file", default="evoclust_rmsd.csv")
    parser.add_argument('-n', "--target_name",
                        help="target name in the alignment, used to map target on the alignment, e.g. target, ade, rp14 etc.", required=True)
    parser.add_argument('-m', "--mapping_fn", help="map folders on the drive with sequence names in the alignment (<name in the alignment>:<folder name>), use | to \
    for multiple seqs, e.g. 'target:rp14_farna_eloop_nol2fixed_cst|AACY023581040:aacy23_cst', use | as a separator", required=True)
    parser.add_argument('files', nargs='+', help='files')
    parser.add_argument('-g', '--group_name',
                        help='name given group of structure, helps to analyze results', default='')
    return parser


def calc_evo_rmsd(targetfn, target_name_alignment, files, mapping, rna_alignment_fn, group_name='', output_fn=None):
    ra = RNAalignment(rna_alignment_fn)
    print('target', targetfn)
    target_residues = ra.get_range(target_name_alignment)
    target = RNAmodel(targetfn, target_residues, save=False, output_dir=None)

    # parse mapping to get models (list models)
    rnastruc = open(mapping).read().replace('\n', '').strip().split(
        '|')  # 'target:rp14_farna_eloop_nol2fixed_cst|X:X'
    print(' # of rnastruc :', len(rnastruc))
    print(' rnastruc:', rnastruc)
    print(' WARNING: if any of your PDB file is missing, check mapping!')
    models = []

    for rs in rnastruc:
        try:
            rs_name_alignment, rs_name_dir = rs.split(':')  # target:rp14_farna_eloop_nol2fixed_cst
        except ValueError:
            # if -m 'tpp|tpp_pdb|CP000050.1/ ..
            # rnastruc: ['tpp', 'tpp_pdb', 'CP000050.1/1019813-1019911:tc5_pdb', 'AE017180.1/640928-641029:tae_pdb', 'BX248356.1/234808-234920:tb2_pdb']
            # Traceback (most recent call last):
            # File "/home/magnus/work/src/evoClustRNA/evoClustRNA.py", line 100, in <module>
            # raise Exception("There is an error in your mapping, check all : and | carefully")
            # Exception: There is an error in your mapping, check all : and | carefully
            raise Exception("There is an error in your mapping, check all : and | carefully")

        # print ' ', rs_name_alignment,'<->', rs_name_dir # AACY023581040 <-> aacy23_cst
        for f in files:
            #
            if rs_name_dir in f:  # rp14_farna_eloop_nol2fixed_cst*pdb
                # print(rs_name_dir)
                # models.extend(get_rna_models_from_dir(input_dir + os.sep + rs_name_dir, ra.get_range(rs_name_alignment), False, False)[:])
                # print(ra.get_range(rs_name_alignment))
                models.append(RNAmodel(f, ra.get_range(rs_name_alignment), False, False))

    data = {'target': [], 'model': [], 'rmsd': [], 'group_name': []}
    for model in models:
        # print
        # print r1.fn, r2.fn, r1.get_rmsd_to(r2)#, 'tmp.pdb')
        rmsd = target.get_rmsd_to(model)  # , 'tmp.pdb')
        # print target, model, rmsd, group_name
        data['target'].append(target)
        data['model'].append(model)
        data['rmsd'].append(rmsd)
        data['group_name'].append(group_name)
    df = pd.DataFrame(data, columns=('target', 'model', 'rmsd', 'group_name'))
    if output_fn:
        df.to_csv(output_fn)
    try:
        df.sort_values(by='rmsd').plot(y='rmsd', use_index=False)
    except TypeError:
        print('Check if the representives have tags, e.g. c1_thf_pk_...')
    plt.savefig(output_fn.replace('.csv', '.png'))
    return df


def test():
    mapping = 'target:rp14_farna_eloop_nol2fixed_cst|AACY023581040:aacy23_cst|AJ630128:aj63_cst'
    x = calc_evo_rmsd("test_data/rp14/rp14_5ddp_bound_clean_ligand.pdb", 'target',
                      ['test_data/rp14/rp14_farna_eloop_nol2fixed_cst/rp14_farna_eloop_nol2fixed_cst.out.1.pdb'],
                      mapping, rna_alignment_fn="test_data/rp14/rp14sub.stk")
    print(x)
    sys.exit(0)


# main
if __name__ == '__main__':
    # if True: test()
    parser = get_parser()
    opts = parser.parse_args()
    df = calc_evo_rmsd(opts.target, opts.target_name, opts.files, opts.mapping_fn,
                       opts.rna_alignment_fn, opts.group_name, opts.output_fn)
    print(df)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_roseta_n.py - show me # of structure in a silent file

Example::

     $ rna_rosetta_n.py ade.out
     21594

"""
from __future__ import print_function
import subprocess
import argparse


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-v', '--verbose', action='store_true')
    parser.add_argument('file', help='ade.out')
    return parser


def get_no_structures(file):
    p = subprocess.Popen('cat ' + file + ' | grep SCORE | wc -l', shell=True,
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stderr = p.stderr.read().strip()
    if stderr:
        print(stderr)
    return int(p.stdout.read().strip()) - 1


def run():
    """Pipline for modeling RNA."""
    args = get_parser().parse_args()
    ns = get_no_structures(args.file)
    if args.verbose:
        print(args.file.ljust(30), end='')
    print(ns)


# main
if __name__ == '__main__':
    run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function
import pandas as pd
import argparse
import os

#plotting inside ipython
import matplotlib.pyplot as plt
import matplotlib
from sklearn.preprocessing import MinMaxScaler

matplotlib.style.use('ggplot')#seaborn-deep')
import pandas as pd

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('csv', help="", default="")
    # parser.add_argument('--sep', help="default is ,; can be also '\t'", default=",")
    # parser.add_argument("-v", "--verbose",
    #action="store_true", help="be verbose")
    parser.add_argument('--plot', help="output plot", default="plot.png")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    df = pd.read_csv(args.csv)
    df = df.sort_values(by=['mqapRNArst'], ascending=[False])

    df['SS'] = df['mqapRNA'] + df['inf'] + df['rst']
    df['RST'] = df['mqapRNA'] + df['rst']

    ax = df.plot(x='fn', y="SS", kind="barh", color="orange")
    ax = df.plot(x='fn', y="RST", kind="barh", ax=ax, color="gray")#, figsize=(10,10));
    ax = df.plot(x='fn', y="mqapRNA", kind="barh", ax=ax, color="darkred")

    ax.set_title('mqapRNA score (the lower the better)')
    plt.grid(True)
    plt.tight_layout()
    plt.savefig(args.plot, dpi=300)



"""show_dists - show distances in PyMOL

.. image:: ../pngs/getdists.png

Usage::

    PyMOL>show_dists([[1,2]])
    1, 2, 3.41

"""


def show_dists(interactions):  # l=([1,2], [3,4])
    for i in interactions:
        a = "////" + str(i[0]) + "/C2"
        b = "////" + str(i[1]) + "/C2"
        x = cmd.distance('d' + str(i[0]) + '-' + str(i[1]), "(" + a + ")", "(" + b + ")")
        print("%i, %i, %s" % (i[0], i[1], str(round(x, 2))))  # why? to have 2.xx
        # mode, 4)


try:
    from pymol import cmd
except:
    print('PyMOL required')
else:
    print('show_dists([[1,2]]) loaded')
    cmd.extend('show_dists', show_dists)



#!/usr/bin/env python
"""random_assignment_of_nucleotides.py - Random assignment of nucleotides for non-typical characters in the sequence alignment (arg --alignfn or fasta file with sequneces (arg --seqfn)

::

    R = G A (purine)
    Y = U C (pyrimidine)
    K = G U (keto)
    M = A C (amino)
    S = G C (strong bonds)
    W = A U (weak bonds)
    B = G U C (all but A)
    D = G A U (all but C)
    H = A C U (all but G)
    V = G C A (all but T)
    N = A G C U (any)

author: A. Zyla - azyla

.. warning:: Tested only on fasta files! and requires Biopython (tested with v1.68)
"""

import logging
import argparse
import random
from Bio import AlignIO
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
from Bio.Seq import Seq
import warnings

# logger
logger = logging.getLogger()
handler = logging.StreamHandler()
logger.addHandler(handler)

def get_align(alignfn):
    """Get seq from an alignment with gaps.
    Args:
       alignfn (str): a path to an alignment
    Usage::
        >>> get_align('test_data/aln1.fasta')
        SingleLetterAlphabet() alignment with 2 rows and 13 columns
        AGGGGGACAGNYU 1
        CYGA------CGG 2
obj1', description='obj1', dbxrefs=[]), id='<unknown id>', name='<unknown name>', description='<unknown description>', dbxrefs=[])
    Returns:
         alignment
    """
    alignment = AlignIO.read(alignfn, 'fasta')
    #alignment = SeqIO.index(alignfn, 'fasta')
    return alignment

def get_sequences(seqfn):
    """Get seq from an fasta file.
    Args:
       seqfn (str): a path to a fasta file
    Usage::
        >>> get_align('test_data/fasta.fasta')

    Returns:
         [SeqRecord(seq=Seq('GGGYYGCCNRW', SingleLetterAlphabet()), id='1', name='1', description='1', dbxrefs=[]), SeqRecord(seq=Seq('GGRGYYGCCUURWAA', SingleLetterAlphabet()), id='1', name='1', description='1', dbxrefs=[])]

    """
    #alignment = AlignIO.read(alignfn, 'fasta')
    alignment = []
    for seq_record in SeqIO.parse(seqfn, 'fasta'):
        alignment.append(seq_record)
        #print alignment
    return alignment

def write_align(align, outfn):
    """Write cleaned alignment with Biopython.
    Args:
        align (obj): a cleaned alignment
        outfn (str): a path to a new alignment file
    Returns:
        none: writes to a file in fasta format
    """
    io = AlignIO
    io.write(alignment, outfn, 'fasta')

    logger.info('Alignment written to %s' % outfn)


def write_seq(seqfn, outfn):
    """Write cleaned alignment with Biopython.
    Args:
        align (obj): a cleaned alignment
        outfn (str): a path to a new alignment file
    Returns:
        none: writes to a file in fasta format
    """
    io = SeqIO
    io.write(alignment, outfn, 'fasta')

    logger.info('Alignment written to %s' % outfn)


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")
    parser.add_argument("--alignfn", help="alignment in the Fasta format")
    parser.add_argument("--seqfn", help="sequences in the Fasta format")
    parser.add_argument("--outfn", help="output aln file (default: alnfn .fasta -> _out.fasta)")
    return parser

# main
if __name__ == '__main__':
    args = get_parser().parse_args()

    if args.verbose:
        logger.setLevel(logging.INFO)

    if not args.outfn:
        if args.alignfn:
            args.outfn = args.alignfn.replace('.fasta', '_out.fasta')

        if args.seqfn:
            args.outfn = args.seqfn.replace('.fasta', '_out.fasta')


    if args.alignfn:
        alignment=get_align(args.alignfn)
        for record in alignment:
            #print(record.seq)
            s = ''
            nseq = ''

            # replace R
            RR = ['R']
            R = ['G', 'A']
            new_R = random.choice(R)

            # replace Y
            YY = ['Y']
            Y = ['U', 'C']
            new_Y = random.choice(Y)

            # replace K
            KK = ['K']
            K = ['G', 'U']
            new_K = random.choice(K)

            # replace M
            MM = ['M']
            M = ['A', 'C']
            new_M = random.choice(M)

            # replace S
            SS = ['S']
            S = ['G', 'C']
            new_S = random.choice(S)

            # replace W
            WW = ['W']
            W = ['A', 'U']
            new_W = random.choice(W)

            # replace B
            BB = ['B']
            B = ['G', 'U', 'C']
            new_B = random.choice(B)

            # replace D
            DD = ['D']
            D = ['G', 'A', 'U']
            new_D = random.choice(D)

            # replace H
            HH = ['H']
            H = ['A', 'C', 'U']
            new_H = random.choice(H)

            # replace V
            VV = ['V']
            V = ['G', 'C', 'A']
            new_V = random.choice(V)

            # replace N
            NN = ['N']
            N = ['G', 'A', 'C', 'U']
            new_N = random.choice(N)

            # replace M
            MM = ['M']
            # replace T
            TT = ['T']

            for i in record.seq:
                if str(i) in RR:
                    i = i.replace('R', new_R)
                    nseq += i
                elif str(i) in YY:
                    i = i.replace('Y', new_Y)
                    nseq += i
                elif str(i) in KK:
                    i = i.replace('K', new_K)
                    nseq += i
                elif str(i) in SS:
                    i = i.replace('S', new_S)
                    nseq += i
                elif str(i) in WW:
                    i = i.replace('W', new_W)
                    nseq += i
                elif str(i) in BB:
                    i = i.replace('B', new_B)
                    nseq += i
                elif str(i) in DD:
                    i = i.replace('D', new_D)
                    nseq += i
                elif str(i) in HH:
                    i = i.replace('H', new_H)
                    nseq += i
                elif str(i) in VV:
                    i = i.replace('V', new_V)
                    nseq += i
                elif str(i) in NN:
                    i = i.replace('N', new_N)
                    nseq += i
                elif str(i) in MM:
                    i = i.replace('M', '-')
                    nseq += i
                elif str(i) in TT:
                    i = i.replace('T', 'U')
                    nseq += i
                else:
                    nseq += i
                seq = nseq  # cleaned sequence
                #
            # print seq, "XXX"
            # print type(record.seq)
            record.seq = Seq(seq)

    if args.seqfn:
        alignment=get_sequences(args.seqfn)

        for seq_record in alignment:
            #print seq_record.seq
            s = ''
            nseq = ''

            #replace R
            RR = ['R']
            R = ['G', 'A']
            new_R = random.choice(R)


            #replace Y
            YY = ['Y']
            Y = ['U', 'C']
            new_Y = random.choice(Y)

            #replace K
            KK =['K']
            K = ['G','U']
            new_K = random.choice(K)

            #replace M
            MM = ['M']
            M = ['A', 'C']
            new_M = random.choice(M)

            #replace S
            SS = ['S']
            S = ['G', 'C']
            new_S = random.choice(S)

            #replace W
            WW = ['W']
            W = ['A', 'U']
            new_W = random.choice(W)

            #replace B
            BB = ['B']
            B = ['G', 'U', 'C']
            new_B = random.choice(B)

            #replace D
            DD = ['D']
            D = ['G', 'A', 'U']
            new_D = random.choice(D)

            #replace H
            HH = ['H']
            H = ['A', 'C', 'U']
            new_H = random.choice(H)

            #replace V
            VV = ['V']
            V = ['G', 'C', 'A']
            new_V = random.choice(V)

            #replace N
            NN = ['N']
            N = ['G', 'A', 'C', 'U']
            new_N = random.choice(N)

            # replace M
            MM = ['M']
            # replace T
            TT=['T']

            for i in seq_record.seq:

                if str(i) in RR:
                    i = i.replace('R', new_R)
                    nseq += i
                elif str(i) in YY:
                    i = i.replace('Y', new_Y)
                    nseq += i
                elif str(i) in KK:
                    i = i.replace('K', new_K)
                    nseq += i
                elif str(i) in SS:
                    i = i.replace('S', new_S)
                    nseq += i
                elif str(i) in WW:
                    i = i.replace('W', new_W)
                    nseq += i
                elif str(i) in BB:
                    i = i.replace('B', new_B)
                    nseq += i
                elif str(i) in DD:
                    i = i.replace('D', new_D)
                    nseq += i
                elif str(i) in HH:
                    i = i.replace('H', new_H)
                    nseq += i
                elif str(i) in VV:
                    i = i.replace('V', new_V)
                    nseq += i
                elif str(i) in NN:
                    i = i.replace('N', new_N)
                    nseq += i
                elif str(i) in MM:
                    i = i.replace('M', '')
                    nseq += i
                elif str(i) in TT:
                    i = i.replace('T', 'U')
                    nseq += i
                else:
                    nseq += i
                seq = nseq  # cleaned sequence
                #print nseq
                #
            # print seq, "XXX"
            # print type(record.seq)
            seq_record.seq = Seq(seq)


    #print type(record.seq)
    #print type(alignment)
    align = alignment

    if args.alignfn:
        write_align(align, args.outfn)

    if args.seqfn:
        write_seq(align, args.outfn)


    print('DONE \nSaved to:', args.outfn)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_ss_get_bps.py - get a list of base pairs for a given "fasta ss" file.

Input file::

    cat ade_ss.fa
    >1y26
    CGCUUCAUAUAAUCCUAAUGAUAUGGUUUGGGAGUUUCUACCAAGAGCCUUAAACUCUUGAUUAUGAAGUG
    (((((((((...((((((.........))))))........((((((.......))))))..)))))))))%

Usage::

    $ rna_ss_get_bps.py ade_ss.fa --offset 12
    [[13, 83], [14, 82], [15, 81], [16, 80], [17, 79], [18, 78], [19, 77], [20, 76], [21, 75], [25, 45], [26, 44], [27, 43], [28, 42], [29, 41], [30, 40], [54, 72], [55, 71], [56, 70], [57, 69], [58, 68], [59, 67]]

Now it also work with pseudoknots.
"""
from __future__ import print_function

import argparse
from rna_tools.SecondaryStructure import parse_vienna_to_pairs


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('file', help="file in the Fasta format")
    parser.add_argument('--offset', help="offset", type=int)
    parser.add_argument("-v", "--verbose", action='count',
                        help="be verbose")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    id, seq, ss = open(args.file).read().strip().split('\n')
    pairs, pairs_pk = parse_vienna_to_pairs(ss)

    if args.offset:
        npairs = []
        for pair in pairs:
            npairs.append([pair[0] + args.offset, pair[1] + args.offset])
        for pair in pairs_pk:
            npairs.append([pair[0] + args.offset, pair[1] + args.offset])
    else:
        npairs = pairs + pairs_pk
    npairs.sort()
    print(npairs)



#!/usr/bin/env python

import sys
from rna_tools.rna_tools_lib import RNAStructure


if __name__ == '__main__':
    files = sys.argv[1:]
    if not files:
        print('rna_pdb_merge_into_one.py test_in/*.pdb')
        sys.exit(1)

    c = 1
    for f in files:
        s = RNAStructure(f)
        print("MODEL        " + str(c))
        print(s.get_text(add_end=False))
        print('ENDMDL')
        c += 1
    print('END')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_rosetta_min.py - a script to do minimization

The script takes the number of structures and the analyzed silence file and does the maths.

Job names will be as your silent file preceding with ~, .e.g ``~tha``.

http://www.sciencedirect.com/science/article/pii/S0076687914000524 ::

    ade$ rna_rosetta_cluster.py ade.out

The first number states how many processors to use for the run, while the second number is 1/6 the total number of previously generated FARNA models. If you are running on a supercomputer that only allows specific multiples of processors, use an appropriate number for the first input.::

    rosetta_submit.py min_cmdline min_out 1 24

rosetta_submit.py min_cmdline min_out [1] [16] The first number states how many processors to use for each line in min_cmdline. Here, enter 1 for the first input so that the total number of processors used will be equal to the number of processors entered with the "-proc" flag in command line [12], above. The second number states the maximum time each job will be allowed to run (walltime). Start the run with the appropriate command listed by the out- put above (e.g., source qsubMPI for the Stampede cluster).

E.g. for 20k silet file, 1/6 will be minimized = 3.3k::

    parallel_min_setup.py -silent rp21cr62.out -tag rp21cr62_min  -proc 200 -nstruct 3200 -out_folder mo -out_script MINIMIZE " -ignore_zero_occupancy false "
    rosetta_submit.py MINIMIZE mo 1 100 m

    [peyote2] rp21 easy_cat.py mo
    Catting into:  rp21_min.out ... from 200 primary files. Found 3200  decoys.

    # on 200 cpus it took around ~30min
"""
from __future__ import print_function
import logging

logging.basicConfig(filename='rna_rosetta_min.log', level=logging.INFO,
                    format="%(asctime)s - %(levelname)s - %(message)s")

import argparse
import os
import glob
import subprocess
import math
import logging
import shutil
import re


def get_no_structures(file):
    """Get a number of structures in a silent file"""
    p = subprocess.Popen('cat ' + file + ' | grep SCORE | wc -l', shell=True,
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stderr = p.stderr.read().strip()
    if stderr:
        print(stderr)
    return int(p.stdout.read().strip()) - 1


def min(silent_file, take_n, cpus, go):
    """Run parallel_min_setup (to MINIMIZE file), rosetta_submit.py, and qsubMINI.

    Fix on the way, qsub files::

        -out:file:silent mo/0/mo/123/tha_min.out -> -out:file:silent mo/123/tha_min.out

    I don't know why mo/0/ is there. I might be because of my changes in rosetta_submit.py (?). """

    #-skip coord_constraints
    # + ' -params_file ' + f.replace('.out', '.params') +
    # ERROR:  -params_file not supported in rna_minimize anymore.
    # ERROR:: Exit from: src/apps/public/farna/rna_minimize.cc line: 156
    # -cst_fa_file
    # glycine_riboswitch_constraints glycine_riboswitch.params-ignore_zero_occupancy false -skip_ coord_constraints
    #cmd = 'extract_pdbs.default.linuxgccrelease -in::file::silent cluster.out'

    logging.info('cpun %i' % cpus)

    # parallel_min_setup
    cmd = "parallel_min_setup.py -silent " + silent_file + " -tag " + silent_file.replace('.out', '') + '_min  -proc ' + str(
        cpus) + ' -nstruct ' + str(take_n) + ' -out_folder mo -out_script MINIMIZE "' + ' -ignore_zero_occupancy false "'
    print(cmd)
    logging.info(cmd)
    os.system(cmd)

    # rosetta_submit
    cmd = "rosetta_submit.py MINIMIZE mo 1 100 m"
    print(cmd)
    logging.info(cmd)
    os.system(cmd)

    # fixing qsub file
    d = "qsub_files"

    for c, qf in enumerate(os.listdir(d), 0):
        qf = d + os.sep + qf
        f = open(qf)
        txt = f.read()
        #x = re.search('-out:file:silent (?P<extra>mo\/\d+)\/min', txt )
        txt = txt.replace('-out:file:silent mo/0/', '-out:file:silent ')

        # fix naming of my jobs
        txt = txt.replace('#$ -N mm0', '#$ -N ~' +
                          silent_file.replace('.out', '').strip() + '_' + str(c))  # to get ~rp06bx571_186

        f.close()

        # write to file
        open(qf, 'w').write(txt)

    if go:
        # run qsubMINI
        logging.info('qsubMINI')
        os.system('chmod +x ./qsubMINI')
        os.system('./qsubMINI')


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help='ade.out')
    parser.add_argument('-g', '--go', action='store_true')
    parser.add_argument('-c', '--cpus', help='default: 200', default=200)
    return parser


def run():
    args = get_parser().parse_args()
    ns = get_no_structures(args.file)
    print('# structures:', ns)
    take_n = int(ns * 0.16)  # 1/6
    min(args.file, take_n, int(args.cpus), args.go)


# main
if __name__ == '__main__':
    run()



"""
flatten_obj.py - Flatten multi-state pymol objects into a single state.

<https://pymolwiki.org/index.php/Flatten_obj>

This is particularly useful for dealing with biological assemblies, which are
loaded as multi-state objects when fetched using `fetch PDBID, type=pdb1`. It
can also be used as a quick way to combine multiple objects without causing
collisions between chain identifiers.

The command re-letters chains to avoid collisions. Older versions of PyMOL
restrict the chain id to a single character, so the script will fail for
assemblies with >62 chains. With more recent versions, this problem is solved
with multi-character chain IDs. Several options are available for how
re-lettering should occur.

Author: Spencer Bliven <spencer.bliven@gmail.com>
Date: October 30, 2015
Version: 1.0
License: Public Domain
"""
from pymol import cmd, stored
import re
try:
    from collections import OrderedDict
    _orderedDict = True
except ImportError:
    _orderedDict = False

# PyMOL 1.7.4 introduces support for multi-letter chains, so we can afford to
# use a smaller alphabet. In earlier versions, use lower-case letters if needed
# (requires running `set ignore_case, 0`)
_long_chains = cmd.get_version()[1] >= 1.74
_default_base = 36 if _long_chains else 62

class OutOfChainsError(Exception):
    def __init__(self,msg):
        self.msg=msg
    def __str__(self):
        return str(self.msg)

class ChainSet(object):
    """
    Base class for various methods to rename chains

    Contains _chains, which maps from the renamed chain to a tuple with the
    original (object,state,chain). All dict-like accessors work on ChainSets,
    e.g.
        chain_set["A"] -> ("obj",1,"A")

    """
    def __init__(self):
        # Use an OrderedDict in Python >= 1.7 for better printing
        if _orderedDict:
            self._chains = OrderedDict()
        else:
            self._chains = dict()

    def map_chain(self, obj, state, origChain ):
        """
        map_chain(string obj,int state, string chain]]) -> string

        Maps a chain letter to a unique chainID. Results are unique within each
        instance, and can be used as keys on this chain set.
        """
        raise NotImplementedError("Base class")

    # delegate most methods to _chains
    def __getattr__(self,at):
        if at in "pop popitem update setdefault".split():
            raise AttributeError("type object '%s' has no attribute '%s'"%(type(self),at))
        return getattr(self._chains,at)
    def __cmp__(self,other):    return self._chains.__cmp__(other)
    def __eq__(self,other):     return self._chains.__eq__(other)
    def __ge__(self,other):     return self._chains.__ge__(other)
    def __gt__(self,other):     return self._chains.__gt__(other)
    def __le__(self,other):     return self._chains.__le__(other)
    def __lt__(self,other):     return self._chains.__lt__(other)
    def __ne__(self,other):     return self._chains.__ne__(other)
    def __len__(self):          return self._chains.__len__()
    def __contains__(self,key): return self._chains.__contains__(key)
    def __getitem__(self,key):  return self._chains.__getitem__(key)
    def __iter__(self):         return self._chains.__iter__()
    def __str__(self):          return str(self._chains)

    @staticmethod
    def _int_to_chain(i,base=_default_base):
        """
        _int_to_chain(int,int) -> str

        Converts a positive integer to a chain ID. Chain IDs include uppercase
        characters, numbers, and optionally lowercase letters.

        i = a positive integer to convert
        base = the alphabet size to include. Typically 36 or 62.
        """
        if i < 0:
            raise ValueError("positive integers only")
        if base < 0 or 62 < base:
            raise ValueError("Invalid base")

        quot = int(i)//base
        rem = i%base
        if rem < 26:
            letter = chr( ord("A") + rem)
        elif rem < 36:
            letter = str( rem-26)
        else:
            letter = chr( ord("a") + rem - 36)
        if quot == 0:
            return letter
        else:
            return ChainSet._int_to_chain(quot-1,base) + letter


class DefaultChainSet(ChainSet):
    """
    Avoids relettering chains if possible. If a chain has been used, uses the
    next available chain letter. Note that this can potentially lead to
    cascading renames, e.g. if chains are sorted alphabetically rather than by
    object.

    Used for rename = 0.
    """
    def __init__(self):
        super(DefaultChainSet,self).__init__()
        self._next_chain = 0
    def map_chain(self, obj, state, origChain ):
        # Keep _next_chain up-to-date
        while ChainSet._int_to_chain(self._next_chain) in self:
            self._next_chain += 1
        # Map this chain
        if origChain in self:
            # Rename
            next_chain = ChainSet._int_to_chain(self._next_chain)
            self._next_chain += 1
        else:
            next_chain = origChain
        self._chains[next_chain] = (obj,state,origChain)
        return next_chain

class SequentialChainSet(ChainSet):
    """
    Renumbers all chains starting at A, continuing through the capital letters
    and numbers, and then adding additional letters through 9999 (the last
    valid chain for mmCIF) and beyond.

    Used for rename=1
    """
    def __init__(self):
        super(SequentialChainSet,self).__init__()
        self._next_chain = 0

    def map_chain(self, obj, state, origChain ):
        next_chain = ChainSet._int_to_chain(self._next_chain)
        self._chains[next_chain] = (obj,state,origChain)
        self._next_chain += 1
        return next_chain

class LongChainSet(ChainSet):
    """
    Uses long strings for the chain names. Chains are renamed like
    "%s_%s_%04d"%(original_chainid,objectname,state).

    Used for rename=2
    """
    def map_chain(self, obj, state, origChain ):
        ch = "%s_%s_%04d"%(origChain,obj,state)
        if ch in self:
            raise ValueError("Duplicate chain %s"%(ch))
        self._chains[ch] = (obj,state,origChain)
        return ch




def flatten_obj(name="",selection="",state=0,rename=0,quiet=1,chain_map=""):
    """
DESCRIPTION

    "flatten_obj" combines multiple objects or states into a single object,
    renaming chains where required

USAGE

    flatten_obj name, selection[, state[, rename[, quiet[, chain_map]]]]

ARGUMENTS

    name = a unique name for the flattened object {default: flat}

    selection = the set of objects to include in the flattening. The selection
        will be expanded to include all atoms of objects. {default: all}

    state = the source state to select. Use 0 or -1 to flatten all states {default: 0}

    rename = The scheme to use for renaming chains: {default: 0}
        (0) preserve chains IDs where possible, rename other chains
            alphabetically
        (1) rename all chains alphabetically
        (2) rename chains using the original chain letter, object name, and state

    quiet = If set to 0, print some additional information about progress and
        chain renaming {default: 1}

    chain_map = An attribute name for the 'stored' scratch object. If
        specified, `stored.<chain_map>` will be populated with a dictionary
        mapping the new chain names to a tuple giving the originated object,
        state, and chainID. {default: ""}

NOTES

    Like the select command, if name is omitted then the default object name
    ("flat") is used as the name argument.

    Chain renaming is tricky. PDB files originally limited chains to single
    letter identifiers containing [A-Za-z0-9]. When this was found to be
    limiting, multi-letter chains (ideally < 4 chars) were allowed. This is
    supported as of PyMOL 1.7. Earlier versions do not accept rename=2, and
    will raise an exception when flattening a structure with more than 62
    chains.

EXAMPLES

    flatten_obj flat, nmrObj
    flatten_obj ( obj1 or obj2 )

SEE ALSO

    split_states

    """

    # arguments

    # Single argument; treat as selection
    if name and not selection:
        selection = name
        name = ""
    # default name and selection
    if not name:
        name = "flat"
    if not selection:
        selection = "(all)"

    state = int(state)
    rename = int(rename)
    quiet = int(quiet)

    # Wrap in extra parantheses for get_object_list
    selection = "( %s )" % selection

    if rename == 0:
        chainSet = DefaultChainSet()
    elif rename == 1:
        chainSet = SequentialChainSet()
    elif rename == 2:
        chainSet = LongChainSet()
    else:
        raise ValueError("Unrecognized rename option (Valid: 0,1,2)")

    metaprefix = "temp" #TODO unique prefix

    # store original value of retain_order, which causes weird interleaving of
    # structures if enabled.
    retain_order = cmd.get("retain_order")
    try:
        cmd.set("retain_order",0)

        # create new object for each state
        for obj in cmd.get_object_list(selection):

            if state <= 0:
                # all states
                prefix = "%s_%s_"%(metaprefix,obj)
                cmd.split_states(obj,prefix=prefix)
            else:
                prefix = "%s_%s_%04d"%(metaprefix,obj,state)
                cmd.create(prefix, obj, state, 1)

        # renumber all states
        statere = re.compile("^%s_(.*)_(\d+)$" % metaprefix) # matches split object names

        warn_lowercase = False

        # Iterate over all objects with metaprefix
        try:
            for obj in cmd.get_object_list("(%s_*)"%(metaprefix) ):
                m = statere.match(obj)
                if m is None:
                    print(("Failed to match object %s" %obj))
                    continue
                origobj = m.group(1)
                statenum = int(m.group(2))

                chains = cmd.get_chains(obj)

                rev_chain_map = {} #old -> new, for this obj only
                for chain in sorted(chains,key=lambda x:(len(x),x)):
                    new_chain = chainSet.map_chain(origobj,statenum,chain)
                    rev_chain_map[chain] = new_chain
                    if not quiet:
                        print(("  %s state %d chain %s -> %s"%(origobj,statenum,chain, new_chain) ))
                    if not _long_chains:
                        if len(new_chain) > 1:
                            raise OutOfChainsError("No additional chains available (max 62).")

                space = {'rev_chain_map':rev_chain_map}
                cmd.alter(obj,"chain = rev_chain_map[chain]",space=space)

            print(("Creating object from %s_*"%metaprefix))
            # Recombine into a single object
            cmd.create(name,"%s_*"%metaprefix)

            # Set chain_map
            if chain_map:
                setattr(stored,chain_map,chainSet)

            # Warn if lowercase chains were generated
            if cmd.get("ignore_case") == "on" and any([c.upper() != c for c in list(chainSet.keys())]):
                print("Warning: using lower-case chain IDs. Consider running the "
                        "following command:\n  set ignore_case, 0" )

        finally:
            # Clean up
            print("Cleaning up intermediates")
            cmd.delete("%s_*"%metaprefix)
    finally:
        # restore original parameters
        print("Resetting variables")
        cmd.set("retain_order",retain_order)


cmd.extend('flatten_obj', flatten_obj)

# tab-completion of arguments
cmd.auto_arg[0]['flatten_obj'] = [ cmd.object_sc, 'name or selection', '']
cmd.auto_arg[1]['flatten_obj'] = [ cmd.object_sc, 'selection', '']




try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
    # sys.exit(0)

def spl_color():
  #cmd.do('color grey50') # PRP8
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_B5zwo")
        cmd.do("color grey60, BRR2_B5zwo")
        cmd.do("color palegreen, LEA1_B5zwo")
        cmd.do("color palegreen, Msl1_B5zwo")
        cmd.do("color slate, SNU114_B5zwo")
        cmd.do("color forest, U2_B5zwo")
        cmd.do("color density, U5_B5zwo")
        cmd.do("color firebrick, U6_B5zwo")
        cmd.do("color brown, U4_B5zwo")
        cmd.do("color black, Intron_B5zwo")
        cmd.do("color grey50, PRP4_B5zwo")
        cmd.do("color grey50, PRP31_B5zwo")
        cmd.do("color grey50, PRP6_B5zwo")
        cmd.do("color grey50, PRP3_B5zwo")
        cmd.do("color grey50, DIB1_B5zwo")
        cmd.do("color grey50, SNU13_B5zwo")
        cmd.do("color grey50, LSM8_B5zwo")
        cmd.do("color grey50, LSM2_B5zwo")
        cmd.do("color grey50, LSM3_B5zwo")
        cmd.do("color grey50, LSM6_B5zwo")
        cmd.do("color grey50, LSM5_B5zwo")
        cmd.do("color grey50, LSM7_B5zwo")
        cmd.do("color grey50, LSM4_B5zwo")
        cmd.do("color grey50, SNU66_B5zwo")
        cmd.do("color grey60, BUD13_B5zwo")
        cmd.do("color palegreen, Cus1_B5zwo")
        cmd.do("color smudge, HSH155_B5zwo")
        cmd.do("color sand, HSH49_B5zwo")
        cmd.do("color grey60, PML1_B5zwo")
        cmd.do("color palegreen, PRP11_B5zwo")
        cmd.do("color palegreen, RDS3_B5zwo")
        cmd.do("color smudge, RSE1_B5zwo")
        cmd.do("color grey60, SNU17_B5zwo")
        cmd.do("color palegreen, Ysf3_B5zwo")
        cmd.do("color grey50, SMB1_1_B5zwo")
        cmd.do("color grey50, SMB1_2_B5zwo")
        cmd.do("color grey50, SMB1_3_B5zwo")
        cmd.do("color grey50, SME1_1_B5zwo")
        cmd.do("color grey50, SME1_2_B5zwo")
        cmd.do("color grey50, SME1_3_B5zwo")
        cmd.do("color grey50, SMX3_1_B5zwo")
        cmd.do("color grey50, SMX3_2_B5zwo")
        cmd.do("color grey50, SMX3_3_B5zwo")
        cmd.do("color grey50, SMX2_1_B5zwo")
        cmd.do("color grey50, SMX2_2_B5zwo")
        cmd.do("color grey50, SMX2_3_B5zwo")
        cmd.do("color grey50, SMD3_1_B5zwo")
        cmd.do("color grey50, SMD3_2_B5zwo")
        cmd.do("color grey50, SMD3_3_B5zwo")
        cmd.do("color grey50, SMD1_1_B5zwo")
        cmd.do("color grey50, SMD1_2_B5zwo")
        cmd.do("color grey50, SMD1_3_B5zwo")
        cmd.do("color grey50, SMD2_1_B5zwo")
        cmd.do("color grey50, SMD2_2_B5zwo")
        cmd.do("color grey50, SMD2_3_B5zwo")
        cmd.do("color grey50, PRP9_B5zwo")
        cmd.do("color grey50, PRP21_B5zwo")
        cmd.do("color grey50, SNU23_B5zwo")
        cmd.do("color grey50, PRP38_B5zwo")
        cmd.do("color grey50, SPP381_B5zwo")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_Ba5gm6")
        cmd.do("color grey60, BRR2_Ba5gm6")
        cmd.do("color dirtyviolet, BUD31_Ba5gm6")
        cmd.do("color raspberry, CEF1_Ba5gm6")
        cmd.do("color orange, CWC15_Ba5gm6")
        cmd.do("color ruby, CWC2_hRBM22_Ba5gm6")
        cmd.do("color violetpurple, CWC21_Ba5gm6")
        cmd.do("color bluewhite, CWC22_Ba5gm6")
        cmd.do("color lightpink, PRP45_Ba5gm6")
        cmd.do("color dirtyviolet, CDC40_Ba5gm6")
        cmd.do("color grey70, PRP19_Ba5gm6")
        cmd.do("color lightblue, PRP46_Ba5gm6")
        cmd.do("color chocolate, SLT11/ECM2_Ba5gm6")
        cmd.do("color grey70, SNT309_Ba5gm6")
        cmd.do("color slate, SNU114_Ba5gm6")
        cmd.do("color brightorange, SYF2_Ba5gm6")
        cmd.do("color brightorange, SYF1_Ba5gm6")
        cmd.do("color forest, U2_Ba5gm6")
        cmd.do("color density, U5_Ba5gm6")
        cmd.do("color firebrick, U6_Ba5gm6")
        cmd.do("color black, Intron_Ba5gm6")
        cmd.do("color yellow, Exon_Ba5gm6")
        cmd.do("color grey60, BUD13_Ba5gm6")
        cmd.do("color rasberry, CLF2_Ba5gm6")
        cmd.do("color palegreen, Cus1_Ba5gm6")
        cmd.do("color grey60, CWC24_Ba5gm6")
        cmd.do("color grey60, CWC27_Ba5gm6")
        cmd.do("color smudge, HSH155_Ba5gm6")
        cmd.do("color sand, HSH49_Ba5gm6")
        cmd.do("color grey60, PML1_Ba5gm6")
        cmd.do("color palegreen, PRP11_Ba5gm6")
        cmd.do("color palegreen, PRP2_Ba5gm6")
        cmd.do("color palegreen, RDS3_Ba5gm6")
        cmd.do("color smudge, RSE1_Ba5gm6")
        cmd.do("color grey60, SNU17_Ba5gm6")
        cmd.do("color palegreen, Ysf3_Ba5gm6")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_C5lj3")
        cmd.do("color dirtyviolet, BUD31_C5lj3")
        cmd.do("color raspberry, CEF1_C5lj3")
        cmd.do("color raspberry, CLF1_C5lj3")
        cmd.do("color orange, CWC15_C5lj3")
        cmd.do("color lightteal, CWC16/YJU2_C5lj3")
        cmd.do("color ruby, CWC2_hRBM22_C5lj3")
        cmd.do("color violetpurple, CWC21_C5lj3")
        cmd.do("color bluewhite, CWC22_C5lj3")
        cmd.do("color deepteal, CWC25_C5lj3")
        cmd.do("color dirtyviolet, ISY1_C5lj3")
        cmd.do("color palegreen, LEA1_C5lj3")
        cmd.do("color palegreen, Msl1_C5lj3")
        cmd.do("color lightpink, PRP45_C5lj3")
        cmd.do("color lightblue, PRP46_C5lj3")
        cmd.do("color chocolate, SLT11/ECM2_C5lj3")
        cmd.do("color slate, SNU114_C5lj3")
        cmd.do("color brightorange, SYF1_C5lj3")
        cmd.do("color forest, U2_C5lj3")
        cmd.do("color density, U5_C5lj3")
        cmd.do("color firebrick, U6_C5lj3")
        cmd.do("color black, Intron_C5lj3")
        cmd.do("color yellow, Exon_C5lj3")
        cmd.do("color grey50, SMB1_1_C5lj3")
        cmd.do("color grey50, SMB1_2_C5lj3")
        cmd.do("color grey50, SME1_1_C5lj3")
        cmd.do("color grey50, SME1_2_C5lj3")
        cmd.do("color grey50, SMX3_1_C5lj3")
        cmd.do("color grey50, SMX3_2_C5lj3")
        cmd.do("color grey50, SMX2_1_C5lj3")
        cmd.do("color grey50, SMX2_2_C5lj3")
        cmd.do("color grey50, SMD3_1_C5lj3")
        cmd.do("color grey50, SMD3_2_C5lj3")
        cmd.do("color grey50, SMD1_1_C5lj3")
        cmd.do("color grey50, SMD1_2_C5lj3")
        cmd.do("color grey50, SMD2_1_C5lj3")
        cmd.do("color grey50, SMD2_2_C5lj3")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_Cs5mps")
        cmd.do("color dirtyviolet, BUD31_Cs5mps")
        cmd.do("color raspberry, CEF1_Cs5mps")
        cmd.do("color raspberry, CLF1_Cs5mps")
        cmd.do("color orange, CWC15_Cs5mps")
        cmd.do("color ruby, CWC2_hRBM22_Cs5mps")
        cmd.do("color violetpurple, CWC21_Cs5mps")
        cmd.do("color bluewhite, CWC22_Cs5mps")
        cmd.do("color lightpink, PRP45_Cs5mps")
        cmd.do("color dirtyviolet, CDC40_Cs5mps")
        cmd.do("color lightblue, PRP46_Cs5mps")
        cmd.do("color chocolate, SLT11/ECM2_Cs5mps")
        cmd.do("color slate, SNU114_Cs5mps")
        cmd.do("color brightorange, SYF2_Cs5mps")
        cmd.do("color brightorange, SYF1_Cs5mps")
        cmd.do("color forest, U2_Cs5mps")
        cmd.do("color density, U5_Cs5mps")
        cmd.do("color firebrick, U6_Cs5mps")
        cmd.do("color grey50, 5EXON_Cs5mps")
        cmd.do("color black, Intron_Cs5mps")
        cmd.do("color yellow, Exon_Cs5mps")
        cmd.do("color grey50, SMB1_Cs5mps")
        cmd.do("color grey50, SME1_Cs5mps")
        cmd.do("color grey50, SMX3_Cs5mps")
        cmd.do("color grey50, SMX2_Cs5mps")
        cmd.do("color grey50, SMD3_Cs5mps")
        cmd.do("color grey50, SMD1_Cs5mps")
        cmd.do("color grey50, SMD2_Cs5mps")
        cmd.do("color grey50, PRP18_Cs5mps")
        cmd.do("color grey50, SLU7_Cs5mps")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_P6exn")
        cmd.do("color dirtyviolet, BUD31_P6exn")
        cmd.do("color raspberry, CEF1_P6exn")
        cmd.do("color raspberry, CLF1_P6exn")
        cmd.do("color orange, CWC15_P6exn")
        cmd.do("color lightteal, CWC16/YJU2_P6exn")
        cmd.do("color ruby, CWC2_hRBM22_P6exn")
        cmd.do("color violetpurple, CWC21_P6exn")
        cmd.do("color bluewhite, CWC22_P6exn")
        cmd.do("color palegreen, LEA1_P6exn")
        cmd.do("color palegreen, Msl1_P6exn")
        cmd.do("color lightpink, PRP45_P6exn")
        cmd.do("color dirtyviolet, CDC40_P6exn")
        cmd.do("color grey70, PRP19_1_P6exn")
        cmd.do("color grey70, PRP19_2_P6exn")
        cmd.do("color grey70, PRP19_3_P6exn")
        cmd.do("color grey70, PRP19_4_P6exn")
        cmd.do("color lightblue, PRP46_P6exn")
        cmd.do("color chocolate, SLT11/ECM2_P6exn")
        cmd.do("color slate, SNU114_P6exn")
        cmd.do("color brightorange, SYF1_P6exn")
        cmd.do("color forest, U2_P6exn")
        cmd.do("color density, U5_P6exn")
        cmd.do("color firebrick, U6_P6exn")
        cmd.do("color black, Intron_P6exn")
        cmd.do("color yellow, Exon_P6exn")
        cmd.do("color grey50, SMB1_1_P6exn")
        cmd.do("color grey50, SMB1_2_P6exn")
        cmd.do("color grey50, SME1_1_P6exn")
        cmd.do("color grey50, SME1_2_P6exn")
        cmd.do("color grey50, SMX3_1_P6exn")
        cmd.do("color grey50, SMX3_2_P6exn")
        cmd.do("color grey50, SMX2_1_P6exn")
        cmd.do("color grey50, SMX2_2_P6exn")
        cmd.do("color grey50, SMD3_1_P6exn")
        cmd.do("color grey50, SMD3_2_P6exn")
        cmd.do("color grey50, SMD1_1_P6exn")
        cmd.do("color grey50, SMD1_2_P6exn")
        cmd.do("color grey50, SMD2_1_P6exn")
        cmd.do("color grey50, SMD2_2_P6exn")
        cmd.do("color grey50, PRP22_P6exn")
        cmd.do("color grey50, PRP18_P6exn")
        cmd.do("color grey50, SLU7_P6exn")
        cmd.do("color grey50, unassigned_P6exn")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_P5ylz")
        cmd.do("color dirtyviolet, BUD31_P5ylz")
        cmd.do("color raspberry, CEF1_P5ylz")
        cmd.do("color raspberry, CLF1_P5ylz")
        cmd.do("color orange, CWC15_P5ylz")
        cmd.do("color ruby, CWC2_hRBM22_P5ylz")
        cmd.do("color violetpurple, CWC21_P5ylz")
        cmd.do("color bluewhite, CWC22_P5ylz")
        cmd.do("color palegreen, LEA1_P5ylz")
        cmd.do("color palegreen, Msl1_P5ylz")
        cmd.do("color lightpink, PRP45_P5ylz")
        cmd.do("color dirtyviolet, CDC40_P5ylz")
        cmd.do("color grey70, PRP19_1_P5ylz")
        cmd.do("color grey70, PRP19_2_P5ylz")
        cmd.do("color grey70, PRP19_3_P5ylz")
        cmd.do("color grey70, PRP19_4_P5ylz")
        cmd.do("color lightblue, PRP46_P5ylz")
        cmd.do("color chocolate, SLT11/ECM2_P5ylz")
        cmd.do("color grey70, SNT309_P5ylz")
        cmd.do("color slate, SNU114_P5ylz")
        cmd.do("color brightorange, SYF2_P5ylz")
        cmd.do("color brightorange, SYF1_P5ylz")
        cmd.do("color forest, U2_P5ylz")
        cmd.do("color density, U5_P5ylz")
        cmd.do("color firebrick, U6_P5ylz")
        cmd.do("color black, Intron_P5ylz")
        cmd.do("color grey50, SMB1_1_P5ylz")
        cmd.do("color grey50, SMB1_2_P5ylz")
        cmd.do("color grey50, SME1_1_P5ylz")
        cmd.do("color grey50, SME1_2_P5ylz")
        cmd.do("color grey50, SMX3_1_P5ylz")
        cmd.do("color grey50, SMX3_2_P5ylz")
        cmd.do("color grey50, SMX2_1_P5ylz")
        cmd.do("color grey50, SMX2_2_P5ylz")
        cmd.do("color grey50, SMD3_1_P5ylz")
        cmd.do("color grey50, SMD3_2_P5ylz")
        cmd.do("color grey50, SMD1_1_P5ylz")
        cmd.do("color grey50, SMD1_2_P5ylz")
        cmd.do("color grey50, SMD2_1_P5ylz")
        cmd.do("color grey50, SMD2_2_P5ylz")
        cmd.do("color grey50, PRP22_P5ylz")
        cmd.do("color grey50, PRP18_P5ylz")
        cmd.do("color grey50, SLU7_P5ylz")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_I5y88")
        cmd.do("color dirtyviolet, BUD31_I5y88")
        cmd.do("color raspberry, CLF1_I5y88")
        cmd.do("color orange, CWC15_I5y88")
        cmd.do("color lightteal, CWC16/YJU2_I5y88")
        cmd.do("color ruby, CWC2_hRBM22_I5y88")
        cmd.do("color deepteal, CWC25_I5y88")
        cmd.do("color black, Intron_2_I5y88")
        cmd.do("color palegreen, LEA1_I5y88")
        cmd.do("color palegreen, Msl1_I5y88")
        cmd.do("color lightpink, PRP45_I5y88")
        cmd.do("color dirtyviolet, CDC40_I5y88")
        cmd.do("color grey70, PRP19_1_I5y88")
        cmd.do("color grey70, PRP19_2_I5y88")
        cmd.do("color grey70, PRP19_3_I5y88")
        cmd.do("color grey70, PRP19_4_I5y88")
        cmd.do("color lightblue, PRP46_I5y88")
        cmd.do("color chocolate, SLT11/ECM2_I5y88")
        cmd.do("color grey70, SNT309_I5y88")
        cmd.do("color slate, SNU114_I5y88")
        cmd.do("color brightorange, SYF2_I5y88")
        cmd.do("color brightorange, SYF1_I5y88")
        cmd.do("color forest, U2_I5y88")
        cmd.do("color density, U5_I5y88")
        cmd.do("color firebrick, U6_I5y88")
        cmd.do("color black, Intron_I5y88")
        cmd.do("color grey50, RNA_I5y88")
        cmd.do("color grey50, cwc23_I5y88")
        cmd.do("color grey50, SPP382_I5y88")
        cmd.do("color grey50, NTR2_I5y88")
        cmd.do("color grey50, PRP43_I5y88")
        cmd.do("color grey50, SMB1_1_I5y88")
        cmd.do("color grey50, SMB1_2_I5y88")
        cmd.do("color grey50, SME1_1_I5y88")
        cmd.do("color grey50, SME1_2_I5y88")
        cmd.do("color grey50, SMX3_1_I5y88")
        cmd.do("color grey50, SMX3_2_I5y88")
        cmd.do("color grey50, SMX2_1_I5y88")
        cmd.do("color grey50, SMX2_2_I5y88")
        cmd.do("color grey50, SMD3_1_I5y88")
        cmd.do("color grey50, SMD3_2_I5y88")
        cmd.do("color grey50, SMD1_1_I5y88")
        cmd.do("color grey50, SMD1_2_I5y88")
        cmd.do("color grey50, SMD2_1_I5y88")
        cmd.do("color grey50, SMD2_2_I5y88")
  if True: # fake if, just a quick hack
        cmd.do("color forest, U2_3jb9")
        cmd.do("color density, U5_3jb9")
        cmd.do("color firebrick, U6_3jb9")
        cmd.do("color black, Intron_1_3jb9")
        cmd.do("color black, Intron_2_3jb9")
        cmd.do("color grey50, Spp42_yPrp8_3jb9")
        cmd.do("color orange, CWF15_yCWC15_3jb9")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hP_6icz")
        cmd.do("color forest, U2_hP_6icz")
        cmd.do("color density, U5_hP_6icz")
        cmd.do("color firebrick, U6_hP_6icz")
        cmd.do("color black, Intron_hP_6icz")
        cmd.do("color grey50, cwc23_hP_6icz")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hBa_6ff7")
        cmd.do("color ruby, CWC2_hRBM22_hBa_6ff7")
        cmd.do("color forest, U2_hBa_6ff7")
        cmd.do("color density, U5_hBa_6ff7")
        cmd.do("color firebrick, U6_hBa_6ff7")
        cmd.do("color black, Intron_hBa_6ff7")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hC_5yzg")
        cmd.do("color ruby, CWC2_hRBM22_hC_5yzg")
        cmd.do("color deepteal, CWC25_hC_5yzg")
        cmd.do("color smudge, PRP16_hDHX38_hC_5yzg")
        cmd.do("color forest, U2_hC_5yzg")
        cmd.do("color density, U5_hC_5yzg")
        cmd.do("color firebrick, U6_hC_5yzg")
        cmd.do("color black, Intron_hC_5yzg")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hX_5xjc")
        cmd.do("color deepteal, CWC25_hX_5xjc")
        cmd.do("color forest, U2_hX_5xjc")
        cmd.do("color density, U5_hX_5xjc")
        cmd.do("color firebrick, U6_hX_5xjc")
        cmd.do("color black, Intron_hX_5xjc")
        cmd.do("color grey50, PRKRIP1_hX_5xjc")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_yCs_5mq0")
        cmd.do("color ruby, CWC2_hRBM22_yCs_5mq0")
        cmd.do("color forest, U2_yCs_5mq0")
        cmd.do("color density, U5_yCs_5mq0")
        cmd.do("color firebrick, U6_yCs_5mq0")
        cmd.do("color grey50, 5EXON_yCs_5mq0")
        cmd.do("color black, Intron_yCs_5mq0")
        cmd.do("color yellow, exon-3_yCs_5mq0")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_yC_5lj5")
        cmd.do("color grey60, BRR2_yC_5lj5")
        cmd.do("color dirtyviolet, BUD31_yC_5lj5")
        cmd.do("color raspberry, CEF1_yC_5lj5")
        cmd.do("color raspberry, CLF1_yC_5lj5")
        cmd.do("color orange, CWC15_yC_5lj5")
        cmd.do("color lightteal, CWC16/YJU2_yC_5lj5")
        cmd.do("color ruby, CWC2_hRBM22_yC_5lj5")
        cmd.do("color violetpurple, CWC21_yC_5lj5")
        cmd.do("color bluewhite, CWC22_yC_5lj5")
        cmd.do("color deepteal, CWC25_yC_5lj5")
        cmd.do("color dirtyviolet, ISY1_yC_5lj5")
        cmd.do("color palegreen, Msl1_yC_5lj5")
        cmd.do("color lightpink, PRP45_yC_5lj5")
        cmd.do("color smudge, PRP16_hDHX38_yC_5lj5")
        cmd.do("color grey70, PRP19_1_yC_5lj5")
        cmd.do("color grey70, PRP19_2_yC_5lj5")
        cmd.do("color grey70, PRP19_3_yC_5lj5")
        cmd.do("color grey70, PRP19_4_yC_5lj5")
        cmd.do("color lightblue, PRP46_yC_5lj5")
        cmd.do("color chocolate, SLT11/ECM2_yC_5lj5")
        cmd.do("color grey70, SNT309_yC_5lj5")
        cmd.do("color slate, SNU114_yC_5lj5")
        cmd.do("color brightorange, SYF1_yC_5lj5")
        cmd.do("color forest, U2_yC_5lj5")
        cmd.do("color density, U5_yC_5lj5")
        cmd.do("color firebrick, U6_yC_5lj5")
        cmd.do("color grey50, 5EXON_yC_5lj5")
        cmd.do("color black, Intron_yC_5lj5")
        cmd.do("color grey50, SMB1_1_yC_5lj5")
        cmd.do("color grey50, SMB1_2_yC_5lj5")
        cmd.do("color grey50, SME1_1_yC_5lj5")
        cmd.do("color grey50, SME1_2_yC_5lj5")
        cmd.do("color grey50, SMX3_1_yC_5lj5")
        cmd.do("color grey50, SMX3_2_yC_5lj5")
        cmd.do("color grey50, SMX2_1_yC_5lj5")
        cmd.do("color grey50, SMX2_2_yC_5lj5")
        cmd.do("color grey50, SMD3_1_yC_5lj5")
        cmd.do("color grey50, SMD3_2_yC_5lj5")
        cmd.do("color grey50, SMD1_1_yC_5lj5")
        cmd.do("color grey50, SMD1_2_yC_5lj5")
        cmd.do("color grey50, SMD2_1_yC_5lj5")
        cmd.do("color grey50, SMD2_2_yC_5lj5")
        cmd.do("color grey50, unassigned_yC_5lj5")
  if True: # fake if, just a quick hack
        cmd.do("color palegreen, LEA1_yPre_6g90")
        cmd.do("color palegreen, Msl1_yPre_6g90")
        cmd.do("color palegreen, Ysf3_yPre_6g90")
        cmd.do("color grey50, SMB1_1_yPre_6g90")
        cmd.do("color grey50, SMB1_2_yPre_6g90")
        cmd.do("color grey50, SME1_1_yPre_6g90")
        cmd.do("color grey50, SME1_2_yPre_6g90")
        cmd.do("color grey50, SMX3_1_yPre_6g90")
        cmd.do("color grey50, SMX3_2_yPre_6g90")
        cmd.do("color grey50, SMX2_1_yPre_6g90")
        cmd.do("color grey50, SMX2_2_yPre_6g90")
        cmd.do("color grey50, SMD3_1_yPre_6g90")
        cmd.do("color grey50, SMD3_2_yPre_6g90")
        cmd.do("color grey50, SMD1_1_yPre_6g90")
        cmd.do("color grey50, SMD1_2_yPre_6g90")
        cmd.do("color grey50, SMD2_1_yPre_6g90")
        cmd.do("color grey50, SMD2_2_yPre_6g90")
        cmd.do("color grey50, unassigned_yPre_6g90")
        cmd.do("color grey51, MUD1_yPre_6g90")
        cmd.do("color orange, SNP1_yPre_6g90")
        cmd.do("color grey53, YHC1_yPre_6g90")
        cmd.do("color grey54, PRP39_yPre_6g90")
        cmd.do("color grey55, PRP42_yPre_6g90")
        cmd.do("color grey56, NAM8_yPre_6g90")
        cmd.do("color grey57, SNU56_yPre_6g90")
        cmd.do("color grey58, LUC7_yPre_6g90")
        cmd.do("color grey59, SNU71_yPre_6g90")
        cmd.do("color grey60, HSH155_yPre_6g90")
        cmd.do("color grey61, RSE1_yPre_6g90")
        cmd.do("color grey62, CUS1_yPre_6g90")
        cmd.do("color grey63, HSH49_yPre_6g90")
        cmd.do("color grey64, RDS3_yPre_6g90")
        cmd.do("color grey65, PRP9_yPre_6g90")
        cmd.do("color grey66, PRP11_yPre_6g90")
        cmd.do("color grey67, PRP21_yPre_6g90")
  if True: # fake if, just a quick hack
        cmd.do("color black, Intron_yE_6n7r")
        cmd.do("color grey50, SMB1_yE_6n7r")
        cmd.do("color grey50, SME1_yE_6n7r")
        cmd.do("color grey50, SMX3_yE_6n7r")
        cmd.do("color grey50, SMX2_yE_6n7r")
        cmd.do("color grey50, SMD3_yE_6n7r")
        cmd.do("color grey50, SMD1_yE_6n7r")
        cmd.do("color grey50, SMD2_yE_6n7r")
        cmd.do("color grey51, MUD1_yE_6n7r")
        cmd.do("color orange, SNP1_yE_6n7r")
        cmd.do("color grey53, YHC1_yE_6n7r")
        cmd.do("color grey54, PRP39_yE_6n7r")
        cmd.do("color grey55, PRP42_yE_6n7r")
        cmd.do("color grey56, NAM8_yE_6n7r")
        cmd.do("color grey57, SNU56_yE_6n7r")
        cmd.do("color grey58, LUC7_yE_6n7r")
        cmd.do("color grey59, SNU71_yE_6n7r")
        cmd.do("color green, U1_yE_6n7r")
  if True: # fake if, just a quick hack
        cmd.do("color black, Intron_yE_6n7p")
        cmd.do("color grey50, SMB1_yE_6n7p")
        cmd.do("color grey50, SME1_yE_6n7p")
        cmd.do("color grey50, SMX3_yE_6n7p")
        cmd.do("color grey50, SMX2_yE_6n7p")
        cmd.do("color grey50, SMD3_yE_6n7p")
        cmd.do("color grey50, SMD1_yE_6n7p")
        cmd.do("color grey50, SMD2_yE_6n7p")
        cmd.do("color grey51, MUD1_yE_6n7p")
        cmd.do("color orange, SNP1_yE_6n7p")
        cmd.do("color grey53, YHC1_yE_6n7p")
        cmd.do("color grey54, PRP39_yE_6n7p")
        cmd.do("color grey55, PRP42_yE_6n7p")
        cmd.do("color grey56, NAM8_yE_6n7p")
        cmd.do("color grey57, SNU56_yE_6n7p")
        cmd.do("color grey58, LUC7_yE_6n7p")
        cmd.do("color grey59, SNU71_yE_6n7p")
        cmd.do("color green, U1_yE_6n7p")
        cmd.do("color grey69, PRP40_yE_6n7p")
        cmd.do("color grey70, STO1_yE_6n7p")
        cmd.do("color grey71, CBC2_yE_6n7p")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_simrna_energy_plot.py - get an energy plot of SimRNA trajectory
"""

from __future__ import print_function
from collections import deque
from rna_tools.tools.simrna_trajectory.simrna_trajectory import SimRNATrajectory
import numpy as np
import gc
import argparse

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("file", help="", default="")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    s = SimRNATrajectory()
    s.load_from_file(args.file, top_level=True)
    #if args.verbose:
    s.plot_energy('plot.png')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function

import argparse

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("-d", "--debug",
                        action="store_true", help="be verbose")
    parser.add_argument('--id-width', type=int, default=70)
    parser.add_argument('--sep', default='|')
    parser.add_argument("alignment")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    name_width = args.id_width

    for l in open(args.alignment):
        if l:
            if args.debug:
                print(l)
            if not l.startswith('#') and not l.startswith('//'):
                if args.debug: print(l.split())
                id, seq = l.split()
                id = id.replace('[', '|').replace(']', '|')
                if args.debug: print(id)
                species, group = ('','')
                if len(id.split('|')) > 2:
                    species, group = id.split('|')[:2]
                    if args.debug: print(group, species)
                    id = group + '|' + species
                else:
                    id = id[0]
                # Leishmania-major-strain-Friedlin[Euglenozoa]FR796420.1|1.0|CUCU-AUG/1-7
                line = id.ljust(name_width) + seq.strip()
                print(line)
            elif '#=GC RF_cons' in l:
                ss = l.replace('#=GC RF_cons', '')
                print('#=GC RF_cons'.ljust(name_width) + ss.strip())
            elif '#=GC SS_cons' in l:
                ss = l.replace('#=GC SS_cons', '')
                print('#=GC SS_cons'.ljust(name_width) + ss.strip())
            else:
                print(l.strip())



from distutils.core import setup

setup(
    name='rna-tools',
    version='3.1.5',
    packages=['', 'rna_tools', 'rna_tools.tools', 'rna_tools.tools.misc', 'rna_tools.tools.rna_bp',
              'rna_tools.tools.diffpdb', 'rna_tools.tools.clanstix', 'rna_tools.tools.plotting',
              'rna_tools.tools.ClashCalc', 'rna_tools.tools.PyMOL4RNA', 'rna_tools.tools.rna_x3dna',
              'rna_tools.tools.clarna_app', 'rna_tools.tools.rna_filter',
              'rna_tools.tools.rna_prediction_significance',
              'rna_tools.tools.rna_rosetta', 'rna_tools.tools.rnakb_utils', 'rna_tools.tools.cluster_load',
              'rna_tools.tools.pdb_formatix', 'rna_tools.tools.pdb_formatix.test',
              'rna_tools.tools.rna_calc_inf', 'rna_tools.tools.pymol_drawing',
              'rna_tools.tools.rna_alignment', 'rna_tools.tools.rna_calc_rmsd',
              'rna_tools.tools.rna_calc_rmsd.lib', 'rna_tools.tools.rna_calc_rmsd.lib.rmsd',
              'rna_tools.tools.rna_helix_vis', 'rna_tools.tools.rna_refinement',
              'rna_tools.tools.extra_functions', 'rna_tools.tools.renum_pdb_to_aln',
              'rna_tools.tools.rna_calc_evo_rmsd', 'rna_tools.tools.simrna_trajectory',
              'rna_tools.tools.rna_calc_rmsd_trafl', 'rna_tools.tools.rna_sali2dotbracket',
              'rna_tools.tools.pdbs_measure_atom_dists', 'rna_tools.tools.rna_convert_pseudoknot_formats',
              'rna_tools.tools.rna_pdb_edit_occupancy_bfactor',
              'rna_tools.tools.rna_pdb_merge_structure_with_fragments'],
    url='https://github.com/mmagnus/rna-tools',
    scripts=['rna_tools/rna_pdb_toolsx.py',
             'rna_tools/tools/rna_calc_rmsd/rna_calc_rmsd.py',
             'rna_tools/tools/rna_calc_rmsd/rna_calc_rmsd_all_vs_all.py',
             'rna_tools/tools/diffpdb/diffpdb.py',
             'rna_tools/tools/clanstix/rna_clanstix.py',
             'rna_tools/rna_dot2ct.py',
             'rna_tools/rna_secondary_structure_prediction.py',
             'rna_tools/tools/rna_prediction_significance/rna_prediction_significance.py',
             'rna_tools/tools/rna_multimodels/rna_pdb_merge_into_one.py',
             'rna_tools/tools/rna_calc_inf/rna_calc_inf.py',
             'rna_tools/tools/rna_convert_pseudoknot_formats/rna_pk_simrna_to_one_line.py',
             'rna_tools/tools/clarna_app/clarna_app.py',
             'rna_tools/tools/rna_helix_vis/rna_helix_vis.py',
             'rna_tools/tools/misc/rna_add_chain.py',
             'rna_tools/tools/rna_sali2dotbracket/rna_sali2dotbracket.py',

             'rna_tools/tools/rna_rosetta/rna_rosetta_run.py',
             'rna_tools/tools/rna_rosetta/rna_rosetta_cluster.py',
             'rna_tools/tools/rna_rosetta/rna_rosetta_min.py',
             'rna_tools/tools/rna_rosetta/rna_rosetta_n.py',
             'rna_tools/tools/rna_rosetta/rna_rosetta_extract_lowscore_decoys.py',
             'rna_tools/tools/rna_rosetta/rna_rosetta_check_progress.py',
             'rna_tools/tools/rna_rosetta/rna_rosetta_head.py',

             'rna_tools/tools/simrna_trajectory/rna_simrna_lowest.py',
             'rna_tools/tools/simrna_trajectory/rna_simrna_extract.py',
             'rna_tools/tools/simrna_trajectory/rna_simrna_cluster.py', 

             'rna_tools/tools/plotting/rna_plot_hist.py',
             'rna_tools/tools/plotting/rna_plot_density.py',

             'rna_tools/tools/simrna_trajectory/rna_simrna_lowest.py',
             'rna_tools/tools/simrna_trajectory/rna_simrna_extract.py',
             'rna_tools/tools/simrna_trajectory/rna_simrna_cluster.py',

             'rna_tools/tools/rna_filter/rna_filter.py',
             'rna_tools/tools/rna_filter/rna_ec2x.py',
             'rna_tools/tools/rna_filter/rna_pairs2SimRNArestrs.py',
             'rna_tools/tools/rna_filter/rna_ss_get_bps.py',
             'rna_tools/tools/rna_filter/rna_pairs_diff.py',

             'rna_tools/tools/rna_alignment/rna_align_seq_to_alignment.py',
             'rna_tools/tools/rna_alignment/rna_align_find_core.py',
             'rna_tools/tools/rna_alignment/rna_align_get_ss_from_stk.py',
             'rna_tools/tools/rna_alignment/rna_align_seq_to_alignment.py',
             'rna_tools/tools/rna_alignment/utils/rna_alignment_get_species.py',
             'rna_tools/tools/rna_alignment/utils/rna_alignment_process_id.py',
             'rna_tools/tools/rna_alignment/utils/rna_alignment_r2r.py',
             'rna_tools/tools/rna_alignment/rna_align_find_seq_in_alignment.py',
             'rna_tools/tools/rna_alignment/rna_align_foldability.py',
             'rna_tools/tools/rna_pdb_edit_occupancy_bfactor/rna_pdb_edit_occupancy_bfactor.py',
             'rna_tools/tools/rna_refinement/rna_refinement.py',
             'rna_tools/rna_simrnaweb_download_job.py',
             'rna_tools/tools/rna_calc_evo_rmsd/rna_calc_evo_rmsd.py',
             'rna_tools/tools/rna_calc_rmsd/rna_calc_rmsd_multi_targets.py',
             'rna_tools/tools/rna_calc_rmsd_trafl/rna_calc_rmsd_trafl.py',
             'rna_tools/tools/rna_calc_rmsd_trafl/rna_cal_rmsd_trafl_plot.py',
             'rna_tools/tools/rna_x3dna/rna_x3dna.py',
             'rna_tools/tools/renum_pdb_to_aln/renum_pdb_to_aln.py',
             'rna_tools/tools/pdbs_measure_atom_dists/pdbs_measure_atom_dists.py',
             'rna_tools/tools/rna_alignment/random_assignment_of_nucleotides.py',
             'rna_tools/tools/rna_seq_search_BLASTn_outfmt-6/select_seq_fromBLAStn_6outfm.py',
             'rna_tools/tools/pefx/pefx.py',
             'rna_tools/tools/misc/translate.py',
             'rna_tools/tools/misc/reverse.py',
             'rna_tools/tools/misc/reverse.py',
             'rna_tools/tools/rna_kd/rna_kd2pkd.py',
             'rna_tools/tools/rna_kd/rna_pkd2kd.py',
             'rna_tools/tools/pymol_color_by_conserv/pymol_color_by_conserv.py',
             'rna_tools/tools/pymol_preview_generator/pymol_preview_generator.py'],
    license='GPLv3',
    author='Marcin Magnus',
    author_email='mag_dex@o2.pl',
    description='a toolbox to analyze structures and simulations of RNA',
    install_requires=[
        'numpy',
        'biopython',
        'forgi',
        'progressbar2',
        'sphinx==1.6.7',
        'sphinx-argparse==0.1.15',
        'sphinx-rtd-theme',
        'sphinxcontrib-napoleon',
        'urllib3',
        'pandas',
        'matplotlib',
        'scipy',
        'python-Levenshtein',
        'versioneer',
      ],
)

import versioneer
setup(version=versioneer.get_version(),
       cmdclass=versioneer.get_cmdclass())



#!/usr/bin/env python

"""Calculate

"Process an alignment in the Stockholm format to get sequences and secondary structures:

Example::

    $ rna_align_distance_to_seq.py test_data/gmp_ref.sto test_data/gmp_ref_distance.csv

       distance                            id
    0      1.00                           gmp
    1      0.69    AE000513.1/1919839-1919923
    2      0.73      BA000004.3/387918-388001
    3      0.69  ABFD02000011.1/154500-154585
    4      0.73      AE015927.1/474745-474827
    5      0.75                AAWL01000006.1
    6      0.72                    AM180355.1
    7      0.72      CP001116.1/102374-102457
    8      0.65    AJ965256.1/1260708-1260792

                                                     seq
    0  -----GCGCGGAAAC-AAUGAUGAAU--GGG-UUUA-AAUUGGGC-...
    1  CUGUCGAAGAGACGC-GAUGAAUCCC--GCC-CUGUAAUUCGGGC-...
    2  AAUCAAUAGGGAAGC-AACGAAGCAU--AGC-CUUU-AUAUGGAC-...
    3  AAAUAUUAUAGAGAU-GUUGAAGUAU--AUU-CUAUUA-UUGGGC-...
    4  AUUUUAAGAGGAAAU-UUUGAACUAU--AUA-CUU--AUUUGGGC-...
    5  --UGCAA-UGGGUGU-GAUGAAGUCC--GGA-CAGUAAUGUGGGC-...
    6  AAUAUUU-UAGAAAC-UGAGAAGUAU--AUC-UUAUUA-UUGGGC-...
    7  AUAACGGCACGAAGC-AAUGAAAUGU--UCG-AUGU-AACCGGGC-...
    8  AAAUUAAGGGGAAGC-GUUGAGCCGC--UAC-CCAU-AUGUGGUUC...

                                                     ss:
    0  (((((((((((.(((.......((((..(((.(................
    1  (((((((((((.(((.......((((..(((.(................
    2  (((((((((((.(((.......((((..(((.(................
    3  (((((((((((.(((.......((((..(((.(................
    4  (((((((((((.(((.......((((..(((.(................
    5  (((((((((((.(((.......((((..(((.(................
    6  (((((((((((.(((.......((((..(((.(................
    7  (((((((((((.(((.......((((..(((.(................
    8  (((((((((((.(((.......((((..(((.(................

"""
from rna_tools.tools.rna_alignment.rna_alignment import clean_seq_and_ss, RNAalignment
import argparse
import pandas as pd

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="an alignment in the Stokholm format, the first seq will be used to calculate distance to (#TODO pick any seq)")
    parser.add_argument('output', help="csv pandas file")
    return parser

if __name__ == '__main__':
    args = get_parser().parse_args()
    a = RNAalignment(args.file)
    target_seq = ''
    df = pd.DataFrame()
    for s in a:
        if 'XX' in s.seq:  # skip line with X
            continue
        if not target_seq:
            target_seq = s.seq
        df = df.append({
            'id': s.id,
            'seq': s.seq,
            'ss:': s.ss,
            'distance': s.get_distance_to(target_seq),
            'length': len(s.seq.replace('-', '')),
            }, ignore_index=True)
    print(df)
    df.to_csv(args.output)



#!/usr/bin/env python

"""rna_filter.py - calculate distances based on given restrants on PDB files or SimRNA trajectories.

The format of restraints::

    (d:A1-A2 < 10.0 1) = if distance between A1 and A2 lower than 10.0, score it with 1

Usage::

    $ python rna_filter.py -r test_data/restraints.txt -s test_data/CG.pdb
     d:A1-A2 10.0 measured: 6.58677550096 [x]
    test_data/CG.pdb 1.0 1 out of 1

    # $ python rna_filter.py -r test_data/restraints.txt -t test_data/CG.trafl
    (d:A1-A2 <  10.0  1)|(d:A2-A1 <= 10 1)
     restraints [('A1', 'A2', '<', '10.0', '1'), ('A2', 'A1', '<=', '10', '1')]

    Frame #1 e:1252.26
      mb for A1 [ 54.729   28.9375  41.421 ]
      mb for A2 [ 55.3425  35.3605  42.7455]
       d:A1-A2 6.58677550096
      mb for A2 [ 55.3425  35.3605  42.7455]
      mb for A1 [ 54.729   28.9375  41.421 ]
       d:A2-A1 6.58677550096
    # this ^ is off right now
"""

from __future__ import print_function
import logging
from rna_tools.rna_tools_logging import logger
from rna_tools.tools.rna_calc_rmsd.lib.rmsd.calculate_rmsd import get_coordinates
from rna_tools.tools.extra_functions.select_fragment import select_pdb_fragment_pymol_style, select_pdb_fragment
from rna_tools.tools.simrna_trajectory.simrna_trajectory import SimRNATrajectory

import argparse
import re
import numpy as np

logger.setLevel(logging.DEBUG)
logger.propagate = False


class RNAFilterErrorInRestraints(Exception):
    pass


def parse_logic(restraints_fn, verbose):
    """Parse logic of restraints.

    Args:
       restraints_nf(string): path to a file with restraints in the rigth format (see below)
       verbose (bool)       : be verbose?

    Format::

        # ignore comments
        (d:A1-A2 <  10.0  1)|(d:A2-A1 <= 10 1)

    Returns:
       list: parse restraints into a list of lists, e.g. [('A9', 'A41', '10.0', '1'), ('A10', 'A16', '10', '1')]

    """

    txt = ''
    with open(restraints_fn) as f:
        for l in f:
            if not l.startswith('#'):
                txt += l.strip()
    if verbose:
        logger.info(txt)
    restraints = re.findall(
        '\(d:(?P<start>.+?)-(?P<end>.+?)\s*(?P<operator>\>\=|\=|\<|\<\=)\s*(?P<distance>[\d\.]+)\s+(?P<weight>.+?)\)', txt)
    return restraints


def parse_logic_newlines(restraints_fn, offset=0, verbose=False):
    """Parse logic of restraints.

    Args:
       restraints_nf(string): path to a file with restraints in the rigth format (see below)
       verbose (bool)       : be verbose?

    Format::

        # ignore comments
        d:Y23-Y69 < 25.0
        d:Y22-Y69 < 25.0
        # d:<chain><resi_A>-<resi_B> <operator> <distance> <weight>; each restraints in a new line

    Raises:
       __main__.RNAFilterErrorInRestraints: Please check the format of your restraints!

    Returns:
       list: parse restraints into a list of lists, e.g. [('A9', 'A41', '10.0', '1'), ('A10', 'A16', '10', '1')]

    """
    restraints = []
    with open(restraints_fn) as f:
        for l in f:
            if l.strip():
                if not l.startswith('#'):
                    if verbose:
                        logger.info(l)
                    restraint = re.findall(
                        'd:(?P<start>.+?)-(?P<end>.+?)\s*(?P<operator>\>\=|\=|\<|\<\=)\s*(?P<distance>[\d\.]+)\s+(?P<weight>.+?)', l)
                    if restraint:
                        # without [0] it is restraints [[('Y23', 'Y69', '<', '25.0', '1')], [('Y22', 'Y69', '<', '25.0', '1')]]
                        # why? to convert 'Y23', 'Y69', '<', '25.0', '1' -> 'Y23', 'Y69', '<', 25.0, 1
                        start = restraint[0][0][0] + str(int(restraint[0][0][1:]) + offset)
                        end = restraint[0][1][0] + str(int(restraint[0][1][1:]) + offset)
                        restraints.append([start, end, restraint[0][1], restraint[0][2],
                                           float(restraint[0][3]), float(restraint[0][4])])

    if len(restraints) == 0:
        raise RNAFilterErrorInRestraints('Please check the format of your restraints!')
    return restraints  # [('A9', 'A41', '10.0', '1'), ('A10', 'A16', '10', '1')]


def get_distance(a, b):
    diff = a - b
    return np.sqrt(np.dot(diff, diff))


def parse_pdb(pdb_fn, selection):
    """
{'A9': {'OP1': array([ 53.031,  21.908,  40.226]), 'C6': array([ 54.594,  27.595,  41.069]), 'OP2': array([ 52.811,  24.217,  39.125]), 'N4': array([ 53.925,  30.861,  39.743]), "C1'": array([ 55.611,  26.965,  43.258]), "C3'": array([ 53.904,  25.437,  43.809]), "O5'": array([ 53.796,  24.036,  41.353]), 'C5': array([ 54.171,  28.532,  40.195]), "O4'": array([ 55.841,  25.746,  42.605]), "C5'": array([ 54.814,  23.605,  42.274]), 'P': array(
    [ 53.57 ,  23.268,  39.971]), "C4'": array([ 55.119,  24.697,  43.283]), "C2'": array([ 54.563,  26.706,  44.341]), 'N1': array([ 55.145,  27.966,  42.27 ]), "O2'": array([ 55.208,  26.577,  45.588]), 'N3': array([ 54.831,  30.285,  41.747]), 'O2': array([ 55.76 ,  29.587,  43.719]), 'C2': array([ 55.258,  29.321,  42.618]), "O3'": array([ 53.272,  24.698,  44.789]), 'C4': array([ 54.313,  29.909,  40.572])}}
    """
    V = {}
    with open(pdb_fn) as f:
        for line in f:
            if line.startswith("ATOM"):
                curr_chain_id = line[21]
                curr_resi = int(line[22: 26])
                curr_atom_name = line[12: 16].strip()
                if selection:
                    if curr_chain_id in selection:
                        if curr_resi in selection[curr_chain_id]:
                            x = line[30: 38]
                            y = line[38: 46]
                            z = line[46: 54]
                            # V.append(np.asarray([x,y,z],dtype=float))
                            if curr_chain_id + str(curr_resi) in V:
                                V[curr_chain_id +
                                    str(curr_resi)][curr_atom_name] = np.asarray([x, y, z], dtype=float)
                            else:
                                V[curr_chain_id + str(curr_resi)] = {}
                                V[curr_chain_id +
                                    str(curr_resi)][curr_atom_name] = np.asarray([x, y, z], dtype=float)
    return V


def check_condition(condition, wight):
    """return True/False, score"""
    pass


def get_residues(pdb_fn, restraints, verbose):
    residues = set()
    for h in restraints:
        a = h[0]
        b = h[1]
        a = a[0] + ':' + a[1:]
        residues.add(a)  # A19
        b = b[0] + ':' + b[1:]
        residues.add(b)
    # set(['A:41', 'A:9', 'A:10', 'A:16'])

    selection = ','.join(residues)
    selection_parsed = select_pdb_fragment(selection, separator=",", splitting="[,:;]")

    residues = parse_pdb(pdb_fn, selection_parsed)

    # get mb
    for r in residues:
        if 'N9' in residues[r]:  # A,G
            residues[r]['mb'] = residues[r]['N9'] - ((residues[r]['N9'] - residues[r]['C6']) / 2)
        else:  # A,G
            residues[r]['mb'] = residues[r]['N1'] - ((residues[r]['N1'] - residues[r]['C4']) / 2)
    for r in residues:
        if verbose:
            logger.info(' '.join(['mb for ', str(r), str(residues[r]['mb'])]))
    return residues


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('-r', "--restraints_fn",
                        dest="restraints_fn",
                        required=True,
                        help="""restraints_fn:
Format:
(d:A9-A41 <  10.0  1)|(d:A41-A9 <= 10 1)
""")

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")

    parser.add_argument('-s', dest="structures", help='structures',
                        nargs='+')  # , type=string)
    parser.add_argument(
        '--offset', help='use offset to adjust your restraints to numbering in PDB files, ade (1y26)'
        'pdb starts with 13, so offset is -12)', default=0, type=int)
    parser.add_argument('-t', dest="trajectory", help="SimRNA trajectory")
    return parser


def calc_dists_for_pdbs(pdb_files, pairs, verbose):
    """
    """
    # h = ('A1', 'A2', '<', '10.0', '1')
    for pdb_fn in pdb_files:
        # logger.info(pdb_fn)
        score = 0
        residues = get_residues(pdb_fn, restraints, verbose)
        good_dists = 0
        for h in pairs:
            dist = get_distance(residues[h[0]]['mb'], residues[h[1]]['mb'])
            # change distance
            ok = '[ ]'
            if dist < h[4]:
                score += h[5]
                ok = '[x]'
                good_dists += 1
            print(' '.join([' d:' + h[0] + '-' + h[1] + ' ' + str(h[4]), 'measured:', str(dist), ok]))
        print(pdb_fn, score / float(len(restraints)), good_dists, 'out of', len(restraints))


# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    pairs = eval(args.pairs)
    calc_dists_for_pdbs(args.structures, pairs, args.verbose)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""This beauty here will go to sali notation and convert it to dotbracket notation.
The file name should be xxxx.sali

Author: Catarina Almeida"""

import re
import string
import sys
import argparse
import os

dic_ss_elements = {'>': ')', '<': '(', '[': '(', ']': ')', '~': '-', '^': '.'}


def repl(m):
    """This function will replace the length of a given string by the correspondent number of dashes.
    The expression ``qwerty`` will be replaced by ``-----``.
    """
    return '-' * len(m.group())


def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('filename', help="file in the Sali format")
    return parser


def convert_sali2dotbracket(fn):
    """The function needs a filename in the Sali format.
    This function will get the secondary structure of the sequence,
    then get its identifier and then the sequence itself.

    **To get the ss**

    The line with the secondary structure is a list and will look like this::

       ['', '', '', '', '', '', '', '', '', '', '--...<<<[[...]]..>>>>', '', '', '\\n']

    In this case, the ss is in the 11th position. But in some files it may be in the 12th, 13th, 10th, etc..

    If the longest element from the list is extracted, then this problem is overcomed.

    The ss will some times have patterns of repeated gaps, which will come in the form of:

    a. x

    b. xnt

    c. ( x )

    With x being any number, from 1 to 1000.
    These must be converted to the correspondent number of gaps (-) in the converted ss.
    This conversion is done by:

    1 - Identifying the pattern with regex

    2 - Replacing it with repl function.

    As such, the following expressions will replace the previously mentioned patterns:

    a. ``re.sub(r'\d*\d', repl, temp)``

    b. ``re.sub(r'\d*\dnt', repl, temp)``

    c. ``re.sub(r'(?P<smthBeautiful>\(\d+\))', repl, temp)``

    **To get the sequence**

    The sequence, much like the ss, can sometimes be in a different position in the list. Like in the ss, the longest element will be selected.
    Also, like in the ss, patterns for repeated gaps appear. So these must also be removed."""
    for line in open(fn):
        if line.startswith(' ') and not re.search('[a-zA-Z]', line):
            # see pydoc # ss #
            lineSplit = line.split(' ')
            ssbefore = max(lineSplit, key=len)
            ssafter = ''
            for x in ssbefore:
                if x in dic_ss_elements:
                    ssafter += dic_ss_elements[x]
                else:
                    ssafter += x
            re.sub(r'(?P<smthBeautiful>\(\d+\))', repl, ssafter)  # (951)
            re.sub(r'\d*\dnt', repl, ssafter)  # 1598nt
            print(ssafter)
        elif line.startswith(' ') or line.startswith('#'):
            True
        else:
            # get the name of the file and add it to the identifier
            nho = os.path.basename(fn).split('.')[0]
            print('>' + nho + '_' + line.split(" ")[0])  # get the identifiers in fasta format
            # see pydoc # sequence #
            temp = line.split(' ')
            sequence = max(temp, key=len)  # the longest element of the list will be the sequence
            print(re.sub(r'\d*\dnt|\d*\d|(?P<smthBeautiful>\(\d+\))', repl, sequence, end=' '))


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    convert_sali2dotbracket(args.filename)



import rna_alignment as ra

if __name__ == '__main__':
    a = ra.RNAalignment('../../input/RF02679_pistol_stockholm.txt')
    print(a)
    print(a.ss_cons_std)
    for s in a:
        print('----------------------------')
        print(s.seq)
        print(s.ss)
        s.remove_gaps()
        print(s.seq)
        print(s.ss)



#!/usr/bin/env python
"""
Parse to BLASTn tabular output format 6 to get sequences in fasta format from database.

BLASTn tabular output format 6 - Column headers:
qseqid **sseqid** pident length mismatch gapopen **qstart** **qend** sstart send evalue bitscore

author: A. Zyla - azyla

.. warning:: Tested only on fasta files! and requires Biopython (tested with v1.68)
"""


import logging
import argparse
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
import warnings
# logger
logger = logging.getLogger()
handler = logging.StreamHandler()
logger.addHandler(handler)


def get_headers(blastn_out6):
    """Parse blastn_out6 to get list of headers and ranges of sequences
    :param blastn_out6: BLASTn tabular output format 6
    :return: list of headers
    """
    headers_list = []
    seq_start_list = []
    seq_end_list = []
    param_list = []

    for line in blastn_out6:
        new_line = line.split('\t')
        #print new_line
        headers_list.append(new_line[1])
        headers_list = list(headers_list)  # you can put a list(set(headers_list)) to remove duplicates
        #print headers_list
        seq_start_list.append(new_line[6])
        seq_start_list=list(seq_start_list)

        seq_end_list.append(new_line[7])
        seq_end_list=list(seq_end_list)
        #print seq_start_list


        param_list = list([headers_list, seq_start_list, seq_end_list])
        #print param_list
    return param_list


0
def get_records(sequences_file, headers,seq_start, seq_end):
    """Get a list of records from fasta sequence file according to list headers and range of sequence

    :param sequences_file: Biopython object file with sequences - SeqIO
    :param headers: List of headers
    :param seq_start: A starting position of matched sequence
    :param seq_end: An ending position of matched sequence
    :return: List of Biopython objects - SeqRecord
    """
    rec_list = []  #list of records
    for record in sequences_file:
        seqid = record.id
        for header,i,j in zip(headers, seq_start, seq_end):
            i = int(i) - 1 # start -1 because python is counting from 0 :)
            j = int(j) - 1 # end
                # print i,j
                # print record.seq
                # record.seq = record.seq[i:j]
            if header.strip() == seqid: #check if IDs are the same
                record.seq=record.seq[i:j] # check a range
                rec_list.append(record)

    return rec_list


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")
    parser.add_argument("Seqfn", help="sequences in the Fasta format. e.q nt_database")
    parser.add_argument("blastn_out6", help="blastn_out6")

    parser.add_argument("--outfn", help="output aln file (default: seqfn .fasta -> _out.fasta)")
    return parser


# main
if __name__ == '__main__':

    args = get_parser().parse_args()

    if args.verbose:
        logger.setLevel(logging.INFO)

    if not args.outfn:
        args.outfn = args.Seqfn.replace('.fasta', '_out.fasta')

    print "If your  database of sequence is big, patience you must have, my dear user :) "

    Seqfn=args.Seqfn
    blastn_out6 = open(args.blastn_out6)
    sequences_file = SeqIO.parse(Seqfn, 'fasta') #load sequences file

    list_of_param = get_headers(blastn_out6)
    #print list_of_param[1]
    headers = list_of_param[0]
    #print headers

    seq_start = list_of_param[1]
    #print seq_start
    seq_end = list_of_param[2]


    sequences = get_records(sequences_file,headers, seq_start,seq_end) #
    if sequences == []:
        print "Warning: No sequences found!"
    else:
        #print sequences
        SeqIO.write(sequences, args.outfn, "fasta")
        print 'DONE \nSaved to:', args.outfn





#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .rnakb_utils import *

def test_gromacs_ready(fn):
    """Test if gromacs ready
    """
    cmd = 'pdb2gmx -ff amber03 -water none -f %s -o query.gro -p query.top' % fn
    os.system(cmd)

def test():
    """Test #1
    """
    fn = 'test_data/1msy_output4_01-000001_AA.pdb'
    f = open(fn).read()
    out = make_rna_rnakb_ready(f)
    o = open('test_output/1msy_output4_01-000001_AA_rnakb_ready.pdb', 'w')
    o.write(out)
    o.close()
    #test_gromacs_ready('tmp.pdb')

def test2():
    """Test #2
    """
    fn = 'test_data/1duq.pdb'
    f = open(fn).read()
    out = make_rna_rnakb_ready(f)
    o = open('test_output/1duq_rnakb_ready.pdb', 'w')
    o.write(out)
    o.close()

def test2b():
    fn = 'test_data/1duq_rpr.pdb'
    f = open(fn).read()
    out = make_rna_rnakb_ready(f)
    o = open('test_output/1duq_rpr_rnakb_ready.pdb', 'w')
    o.write(out)
    o.close()
    #test_gromacs_ready(fn)

def test3():
    """Test #3
    """
    fn = 'test_data/cat_chunk003_2r8s_5kcycles_decoys_nonativefrags.cluster1.0_clean_error_fx.pdb'
    f = open(fn).read()
    out = make_rna_rnakb_ready(f)
    o = open('test_output/cat_chunk003_2r8s_5kcycles_decoys_nonativefrags.cluster1.0_clean_error_fx_rnakb_ready.pdb', 'w')
    o.write(out)
    o.close()
    #test_gromacs_ready('tmp3.pdb')

def test4():
    """Test #3
    """
    fn = 'test_data/3nt_edited.pdb'
    f = open(fn).read()
    out = make_rna_rnakb_ready(f)
    o = open('test_output/3nt_edited_rnakb_ready.pdb', 'w')
    o.write(out)
    o.close()
    #test_gromacs_ready('tmp3.pdb')

if __name__ == '__main__':
    test()
    test2()
    test2b()
    test3()
    test4()



import unittest
import subprocess

class test_sh(unittest.TestCase):
    """Super-dirty test of test.sh"""
    def test(self):
        cmd = './test.sh'
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        err = p.stderr.read()
        self.assertEqual(err, '')

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Set of functions for working with single lines from PDB files
"""

def get_atom_num(line):
    """Extract atom number from a line of PDB file

    Arguments:
      * line = ATOM line from a PDB file

    Output:
      * atom number as an integer
    """
    return int(''.join([x for x in line[6:10] if x.isdigit()]))


def get_res_num(line):
    """Extract residue number from a line of PDB file

    Arguments:
      * line = ATOM line from a PDB file

    Output:
      * residue number as an integer
    """
    return int(''.join([x for x in line[22:27] if x.isdigit()]))


def get_res_code(line):
    """Get residue code from a line of a PDB file
    """
    if not line.startswith('ATOM'):
        return None
    return line[17:20]


def get_atom_code(line):
    """Get atom code from a line of a PDB file
    """
    if not line.startswith('ATOM'):
        return None
    return line[13:16].replace(' ', '')

def get_atom_coords(line):
    """Get atom coordinates from a line of a PDB file
    """
    if not line.startswith('ATOM'):
        return None
    return tuple(map(float, line[31:54].split()))


def set_line_bfactor(line, bfactor):
    if not line.startswith('ATOM'):
        return None
    return line[:60] + (" %5.2f" % bfactor) + line[66:]


def set_atom_code(line, code):
    return line[:13] + code + ' ' * (3 - len(code)) + line[16:]


def set_res_code(line, code):
    return line[:18] + code.rjust(3) + line[21:]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function
from Seq import RNASequence
import argparse
import sys
import subprocess
import os
reload(sys)
sys.setdefaultencoding('utf8')

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('--method', help="cyclefold, mcfold", default="cyclefold")#, required=True)
    parser.add_argument("--seq", help="", default="")
    parser.add_argument("--file", help="", default="")
    parser.add_argument("--cstinfile", action="store_true", help="the 3rd file should have cst line, e.g. '((....))'")
    parser.add_argument("--cst", help="--cst '((....))'")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    return parser


def exe(cmd):
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err


def exe2(cmd):
    os.system(cmd)


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    cst = args.cst

    if args.seq:
        seq = args.seq

    if args.file:
        f = open(args.file)
        header = f.readline()
        seq = f.readline().strip()
        if args.cstinfile:
            cst = f.readline().strip()
            assert cst, 'Cst line empty, check the input file'

    try:
        print(seq)
    except NameError:
        parser.print_usage()
        parser.exit(1)

    if args.cst:
        cst = args.cst
    if cst:
        print(cst, '<= cst')

    if args.method == 'cyclefold':
        seq = RNASequence(seq)
        ss, en = seq.predict_ss('rnastructure_CycleFold', verbose=args.verbose)
        print(en, ss)

    if args.method == 'mcfold':
        seq = RNASequence(seq)
        if cst:
            en, ss = seq.predict_ss('mcfold', constraints=cst, verbose=args.verbose)
        else:
            en, ss = seq.predict_ss('mcfold', verbose=args.verbose)
        #if not ss:
        #    ss = 'x'
        print(ss, en)



#!/usr/bin/env python

from __future__ import print_function
from collections import OrderedDict
import re
import string
import sys

def select_pdb_fragment(txt, separator="-", splitting='[:\+]', verbose=False):
    """Take txt such as ``A:1-31+B:1-11`` and parse into::

          OrderedDict([('A', [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 
          15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31]),
          ('B', [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11])])          
    
      .. warning:: e.g. for A:1-31, resi 31 is included"""
    txt = txt.replace(' ','')
    if verbose: print(txt)
    #l = re.split, txt)
    l = re.split(splitting, txt)
    if verbose: print(l)

    selection = OrderedDict()
    for i in l: # ['A', '1-10', '15', '25-30', 'B', '1-10']
        if i in string.ascii_letters:
            if verbose: print('chain', i)
            chain_curr = i
            continue

        if i.find(separator) > -1:
            start, ends = i.split(separator)
            start = int(start)
            ends = int(ends)
            if start > ends:
                print('Error: range start > end ' + i, file=sys.stderr)
                sys.exit(1)
            index = list(range(int(start), int(ends)+1)) # without +1 python like, with +1 people-like
        else:
            index=[int(i)]
        if chain_curr in selection:
            selection[chain_curr] += index
        else:
            selection[chain_curr] = index
    return selection


def select_pdb_fragment_pymol_style(txt, verbose=False):
    """Take txt such as A/10-15/P and parse into::
    
      A/57/O2' -> ['A', ['57'], "O2'"]

    If you want to combine a few subselections, please use ``,``::
 
       --model_ignore_selection "A/57/O2',A/58/O2'"
    
    .. warning:: e.g. for A:1-31, resi 31 is included"""
    v = 0
    selection = OrderedDict()
    if txt.find(',') > -1:
        txt = txt.replace(' ','').split(',')
    else:
        txt = [txt]
    for t in txt:
        l = t.split('/')
        if v:print(l)
        l[1] = l[1].split('+')

        if l[0] in string.ascii_letters:
            chain_curr = l[0]

        for i in l[1]:
            if i.find('-') > -1:
                start, ends = i.split('-')
                if start > ends:
                    print('Error: range start > end ' + i, file=sys.stderr)
                    return False
                index = list(range(int(start), int(ends)))#+1)
            else:
                index=[int(i)]

            index_and_atoms = [index, l[2].split('+')]
            if v: print(index_and_atoms)

            if chain_curr in selection:
                selection[chain_curr] += [index_and_atoms]
            else:
                selection[chain_curr] = [index_and_atoms]

    if verbose:
        print(selection)
    return selection

def is_in_selection(selection, curr_chain_id, curr_resi, curr_atom_name):
    if curr_chain_id in selection:
            for sele_range in selection[curr_chain_id]:
                if curr_resi in sele_range[0]:
                        if curr_atom_name in sele_range[1]:
                            return True
    return False                  

#main
if __name__ == '__main__':
    selection = select_pdb_fragment_pymol_style('E/1-15+18/P, A/1-3/P')

    curr_chain_id = 'E'
    curr_resi = 1
    curr_atom_name = 'P'

    print(selection)
    if selection:
        print(is_in_selection(selection, curr_chain_id, curr_resi, curr_atom_name))

    print(is_in_selection(selection, 'X', curr_resi, curr_atom_name))        
    print(is_in_selection(selection, 'E', curr_resi, "C'"))        
    print(is_in_selection(selection, 'A', curr_resi, "P'"))        

    print(select_pdb_fragment_pymol_style('A/48/OP2,B/48/OP2'))

    print(select_pdb_fragment('A:1-31+B:1-11'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from rna_tools.tools.pdb_formatix.SingleLineUtils import get_res_code, get_res_num, get_atom_code, \
    set_atom_code, set_line_bfactor
import re


class PDBFile(object):
    """Class for holding data from a PDB file and modifying it.
    """

    # find 'ATOM' lines in a PDB file
    ATOM_LINE_PATTERN = re.compile('^ATOM')

    # dictionary for changing residue names from 3 letters to 1 letter
    RES3TO1 = {
        'GUA': 'G',
        'URI': 'U',
        'CYT': 'C',
        'ADE': 'A',

        'RG': 'G',
        'RU': 'U',
        'RC': 'C',
        'RA': 'A',

        'R3G': 'G',
        'R3U': 'U',
        'R3C': 'C',
        'R3A': 'A',

        'R5G': 'G',
        'R5U': 'U',
        'R5C': 'C',
        'R5A': 'A',

        'RG3': 'G',
        'RU3': 'U',
        'RC3': 'C',
        'RA3': 'A',

        'RG5': 'G',
        'RU5': 'U',
        'RC5': 'C',
        'RA5': 'A',

        'RGU': 'G',
        'URA': 'U',
        'RCY': 'C',
        'RAD': 'A',

        # just in case
        'G': 'G',
        'U': 'U',
        'C': 'C',
        'A': 'A',
    }

    AMINOACID_CODES = ("Ala", "Arg", "Asn", "Asp", "Cys", "Glu", "Gln", "Gly",
                       "His", "Ile", "Leu", "Lys", "Met", "Phe", "Pro", "Ser", "Thr",
                       "Trp", "Tyr", "Val",
                       "ALA", "ARG", "ASN", "ASP", "CYS", "GLU", "GLN", "GLY",
                       "HIS", "ILE", "LEU", "LYS", "MET", "PHE", "PRO", "SER", "THR",
                       "TRP", "TYR", "VAL"
                       )

    def __init__(self, pdb_string=None, pdb_path=None, pdb_handle=None, verbose=False):
        """Constructor, should get exactly one of the arguments:
             * pdb_string = PDB as a string
             * pdb_path = string with path to a PDB file
             * pdb_handle = handle to a PDB file
        """
        self.verbose = verbose

        if len([x for x in [pdb_string, pdb_path, pdb_handle] if x is not None]) > 1:
            print('You should provide at most one source for PDB file')
            raise Exception
        input_string = ''
        if pdb_string is not None:
            input_string = pdb_string
        elif pdb_path is not None:
            with open(pdb_path) as f:
                input_string = f.read()
        elif pdb_handle is not None:
            input_string = pdb_handle.read()
        self.pdb_string = input_string
        self.pdb_lines = self.pdb_string.split('\n')
        self.fixes = []

    def save(self, file_path):
        """Save current PDB to disk

        Arguments:
          * file_path = path where it will be saved
        """
        with open(file_path, 'w') as f:
            f.write(self.pdb_string)

    def _apply_fix(self, fix_name, result_string, result_lines=None):
        """Helper function for applying fixes and saving information about them.

        Arguments:
          * fix_name = string that will be added to self.pdb_fixes, unless
            result_string is the same as pdb_string
          * result_string = string after applying this fix
          * result_lines = optional, list of lines, use this argument if you
            already have such list and don't want to lose time on splitting
            result_string
        """
        if self.pdb_string != result_string:
            self.fixes.append(fix_name)
            self.pdb_string = result_string
            if result_lines is not None:
                self.pdb_lines = result_lines
            else:
                self.pdb_lines = self.pdb_string.split('\n')

    def set_string(self, pdb_string):
        """Change PDB string stored in this instance of the class

        Arguments:
          * pdb_string = new PDB string
        """
        self.pdb_string = pdb_string
        self.pdb_lines = pdb_string.split('\n')

    def _get_atom_lines(self):
        """Get only lines with ATOM information
        """
        return [l for l in self.pdb_lines if re.match(self.ATOM_LINE_PATTERN, l)]

    def validate_pdb(self):
        """Check if file is a PDB structure

        Output:
          * True if it is a PDB, False otherwise
        """
        atom_lines = self._get_atom_lines()
        if not len(atom_lines):
            return False
        return True

    def detect_proteins(self):
        """Check if there are any aminoacid fragments in the file

        Output:
          * True if there are some, False otherwise
        """
        for l in self.pdb_lines:
            if get_res_code(l) in self.AMINOACID_CODES:
                return True
        return False

    def seq_from_pdb(self):
        """Extract sequence from a PDB and return it as a string.

        Output:
          * sequence, returned as a string
        """
        atoms = [l.split() for l in self._get_atom_lines()]
        if atoms[0][3][0] != 'r':  # it is 'r' if it's a ROSETTA PDB
            seq = [self.RES3TO1[atoms[0][3]]]
        else:
            seq = [atoms[0][3][1]]
        for a in range(1, len(atoms)):
            # atom number is different than previous one
            if atoms[a][5] != atoms[a - 1][5]:
                if atoms[a][3][0] != 'r':  # check for ROSETTA PDB
                    seq.append(self.RES3TO1[atoms[a][3]])
                else:
                    seq.append(atoms[a][3][1])
        return ''.join(seq)

    def seq_from_amber_like_pdb(self):
        """Extract sequence from a PDB and return it as a string - use it for amber-like files.

        Output:
          * sequence, returned as a string, such as: RG5 RC RU RG RG RG RC RG RC RA RG RG3 RC5 RC RU RG RA RC RG RG RU RA RC RA RG RC3
        """
        atoms = [l.split() for l in self._get_atom_lines()]
        seq = []
        seq.append(atoms[0][3])
        for a in range(1, len(atoms)):
            # atom number is different than previous one
            if self.verbose:
                print((atoms[a][5], atoms[a - 1][5]))
            if atoms[a][5] != atoms[a - 1][5]:
                seq.append(atoms[a][3])
        return ' '.join(seq)

    def get_fasta(self, name='seq', lowercase=False):
        """Format sequence in FASTA format, with a header and lines split
        at 80 characters.

        Arguments:
          * name = name of the sequence (it's put in the header line)

        Output:
          * FASTA returned as a string
        """
        seq = self.seq_from_pdb()
        if lowercase:
            seq = seq.lower()
        result = ['>' + name]
        result.extend([seq[i:i + 80] for i in range(0, len(seq), 80)])
        result.append('')
        return '\n'.join(result)

    def remove_non_atoms(self):
        """Remove all lines that are not ATOMs
        """
        result = self._get_atom_lines()
        self._apply_fix('Removed non-atom lines', '\n'.join(result), result)

    def _check_resname_3(self):
        """Check if PDB uses 3 or 1 letter residue names.

        Output:
          * bool, True if 3 letters, False otherwise
        """
        for l in self.pdb_lines:
            if l.startswith('ATOM'):
                return 3 == len(l.split()[3])

    def _resname_3to1(self):
        """Convert residue names in PDB file from 3 letters to 1 letter.
        """
        result = []
        for l in self.pdb_lines:
            if l.startswith('ATOM'):
                long_name = l.split()[3]
                try:
                    short_name = self.RES3TO1[long_name]
                except KeyError:
                    short_name = 'X'
                result.append(l[:17] + '  ' + short_name + l[20:])
            else:
                result.append(l)
        self._apply_fix('resname_3to1', '\n'.join(result), result)

    def resname_check_and_3to1(self):
        """Check if resnames are 3 letter long and if so convert them to 1 letter.
        """
        if self._check_resname_3():
            try:
                self._resname_3to1()
            except:
                print('Conversion to 1-letter residue names failed')

    def terminate_chains(self):
        """Add 'TER' at the end of chain if none 'TER's are found in PDB.
        """
        for l in self.pdb_lines:
            if l.startswith('TER'):
                return
        else:
            result = []
            chain_started = False
            ter_added = False
            for l in self.pdb_lines:
                if 'ATOM' in l:
                    chain_started = True
                else:
                    if chain_started:
                        result.append('TER')
                        ter_added = True
                        chain_started = False
                result.append(l)
        if not ter_added:
            result.append('TER')
        self._apply_fix('terminate_chains', '\n'.join(result), result)

    def _split_by_ters(self, separator='TER\n'):
        """Split a PDB string by TER lines or other separator

        Arguments:
          * separator = optional, separator dividing structures, default is 'TER\n'
        """
        return [i + separator for i in self.pdb_string.split(separator)]

    def remove_short_chains(self, threshold=9):
        """Remove chains that have chains with a small number of atoms.

        Arguments:
          * threshold = if chain has more atoms, it stays in the result
        """
        chains = self.pdb_string.split('TER\n')
        good_chains = [c for c in chains if len(c.split('\nATOM')) > threshold]
        self._apply_fix('remove_short_chains', '\n'.join(good_chains))

    def check_and_add_P_at_start(self):
        residues = {}
        for l in self.pdb_lines:
            res_num = get_res_num(l)
            if res_num in residues:
                residues[res_num].append(l)
            else:
                residues[res_num] = [l]
        first_residue = residues[min(residues.keys())]
        # check P
        has_p = False
        for l in first_residue:
            if get_atom_code(l) == 'P':
                has_p = True
        if not has_p:
            # add P
            corrected_residue = []
            for l in first_residue:
                if get_atom_code(l) == 'O5\'' or get_atom_code(l) == 'O5*':
                    corrected_residue.append(set_atom_code(l, 'P'))
                else:
                    corrected_residue.append(l)
            residues[min(residues.keys())] = corrected_residue
            residues = ['\n'.join(residues[r]) for r in residues]
            self._apply_fix('add_P_at_start', '\n'.join(residues))

    def set_residues_bfactor(self, bfactors):
        """Set B-factor to any value you want

        Arguments:
          * bfactors = list of B-factors that will be set, should be of same
            length as nucleotide sequence
        """
        ans = []
        for l in self.pdb_lines:
            if l.startswith('ATOM'):
                res_num = get_res_num(l)
                try:
                    ans.append(set_line_bfactor(l, bfactors[res_num - 1]))
                except IndexError:
                    pass
            else:
                ans.append(l)
        self.pdb_lines = ans
        self.pdb_string = '\n'.join(ans)

    def pedantic_pdb(self):
        """Do everything that's possible to fix PDB: 3-to-1, no HETATMs, TERs etc.
        """
        self.resname_check_and_3to1()
        self.terminate_chains()
        self.remove_short_chains()

    def count_models(self):
        """Count models in the PDB file

        Output:
          * number of models as an int
        """
        model_num = 0
        for l in self.pdb_lines:
            if l.startswith('MODEL'):
                model_num += 1
        return model_num

    def get_model(self, model_num):
        """Get n-th model from the file

        Arguments:
          * model_num = number of model to get, starts with 0
        """
        model_borders = list(zip(
            [i[0] for i in enumerate(self.pdb_lines) if i[1].startswith('MODEL')],
            [i[0] for i in enumerate(self.pdb_lines) if i[1].startswith('ENDMDL')]
        ))
        result = self.pdb_lines[:model_borders[0][0]]
        result.extend(self.pdb_lines[model_borders[model_num][0]:model_borders[model_num][1] + 1])
        result.extend(self.pdb_lines[model_borders[-1][1] + 1:])
        self._apply_fix('get_model', '\n'.join(result), result)

    def check_and_get_first_model(self):
        """Check if there are more than one models and get only the first one
        """
        if self.count_models() >= 2:
            self.get_model(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
 ScenePNG: wrote 2052x1350 pixel image to file "out.png".
(base) [mx] structures-sessions$ pymol -c /Users/magnus/work/cwc15/structures-sessions/cwc15_conserv.pse  -d 'ray; png out.png; quit'
"""
import sys
import argparse
import tempfile
import os
import subprocess


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("-b", "--black",
                        action="store_true", help="black bg")
    parser.add_argument("files", help="", default="", nargs='+')
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    for file in args.files:
        tf = tempfile.NamedTemporaryFile(delete=False)
        f = tf.name + '.png'
        file = file.replace(" ", "\\ ")
        os.system('/usr/local/bin/pymol -c ' + file + " -d 'set ray_opaque_background, off; save " + f + "; quit'") #  ray 300,300,renderer=0 ray 800, 800;
        if args.verbose:
            print(f)
        # crop
        fcrop = f.replace('.png', '32.png')
        cmd = "/usr/local/bin/convert " + f + " -gravity center -crop 3:3 +repage " + fcrop
        os.system(cmd)
        #cmd = 'source activate base && /usr/local/bin/fileicon set ' + args.file + ' ' + f
        cmd = 'unset PYTHONPATH && /usr/local/bin/fileicon set ' + file + ' ' + fcrop
        os.system(cmd)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
import subprocess
import doctest_cmds

def run_cmd(cmd, verbose=True):
    if verbose:
        print(cmd)
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err


def is_modifed(fn):
    out, err = run_cmd('git status %s' % fn)
    if 'modified' in out or 'untracked file' in out:
        return True
    else:
        return False


def test_rna_filter():
    fn = "test_data/rna_filter.txt"
    cmd = "python rna_filter.py -r test_data/restraints.txt -s test_data/CG.pdb | " \
          "tee test_data/rna_filter.txt"
    out, err = run_cmd(cmd)
    assert is_modifed(fn) is False



def test_rna_filter_cmd():
    assert doctest_cmds.is_ok('rna_filter.py') is True

def test_rna_pairs_diff():
    assert doctest_cmds.is_ok('rna_pairs_diff.py') is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""select top interactions

I implemented correction to sync these results with excel files from the Marks lab.
"""
from __future__ import print_function
import argparse
import pandas as pd
import sys


def sortdata(filename):
    df = pd.read_csv(filename, sep=" ", names=['i', ' ', 'j', ' ', ' ', 'scores'])
    df2 = df.sort_values(by=['scores'], ascending=False)
    maxvalue = max(df2[['j']].values)
    l = int(maxvalue)
    print('l', l)
    L = l / 2
    #L = l
    print('L', L)
    df3 = df2[0:L]
    df4 = df3[['i', 'j', 'scores']]
    # correction for -1 ?
    df4['i'] -= 1
    df4['j'] -= 1

    print(df4)
    csvfn = filename + "_" + str(L) + "_selected.csv"
    print('Output file created: ' + csvfn)
    df4.to_csv(csvfn, sep=" ", index=False)


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('file', help="plm scores, direct output from plm", default="")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    sortdata(args.file)



from simrna_trajectory import SimRNATrajectory


def test():
    s = SimRNATrajectory()
    s.load_from_file('test_data/mini.trafl', top_level=True)
    s.plot_energy('plot.png')
    print('OK')

if __name__=="__main__":
    test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    import urllib3
except:
    print('BlastPDB requires urllib3')


class BlastPDB:
    """BlastPDB - run Blast online on the PDB database.

    This can be used in Jupiter based RNA notebooks, e.g.
    https://github.com/mmagnus/rna-pdb-tools/blob/master/rp18.ipynb

    Usage::

       >>> p = BlastPDB('GGGUCAGGCCGGCGAAAGUCGCCACAGUUUGGGGAAAGCUGUGCAGCCUGUAACCCCCCCACGAAAGUGGG')
       >>> p.search()
       >>> p.result  #doctest: +ELLIPSIS
       u'<HTML>\\n<TITLE>BLAST Search Results</TITLE>...

    :param seq: string
    """

    def __init__(self, seq):
        self.seq = seq
        self.result = ''

    def search(self):
        """Search online the seq."""
        http = urllib3.PoolManager()
        response = http.request('GET', 'http://www.rcsb.org/pdb/rest/getBlastPDB1?sequence=' +
                                self.seq +
                                '&eCutOff=10.0&matrix=BLOSUM62&outputFormat=HTML')
        if response.status == 200:
            self.result = response.data.decode()


# main
if __name__ == '__main__':
    import doctest
    doctest.testmod()



import matplotlib.pyplot as plt  # to start
import pandas as pd
import matplotlib.pyplot as plt
plt.style.use('classic')
import numpy as np
import sys

"""version when the same protein gets SMD2_2 and SMD2_1"""

f = "/Users/magnus/work-src/rna-tools/rna_tools/tools/PyMOL4RNA/Spliceosome_PyMOL.xlsx"
df = pd.read_excel(f)

strucs = [
          ['S1_B_5zwo', '5zwo', '_B5zwo'],
          ['S2_Bact_5gm6', '5gm6', '_Ba5gm6'],
          ['S3_C_5lj3', '5lj3', '_C5lj3'],
          ['S4_Cstar_5mps', '5mps', '_Cs5mps'],
          ['S5_P_6exn', '6exn', '_P6exn'],
          ['S5_P_5ylz', '5ylz', '_P5ylz'],
          ['S6_ILS_5y88', '5y88', '_I5y88'],
          ['SX_pX_3jb9', '3jb9', '_3jb9'],
          ['S6_hP_6icz', '6icz', '_hP_6icz'],
          ['hBa_6ff7', '6ff7', '_hBa_6ff7'],
          ['hC_5yzg', '5yzg', '_hC_5yzg'],
          ['hX_5xjc', '5xjc', '_hX_5xjc'],
          ['yCs_5mq0', '5mq0', '_yCs_5mq0'],
          ['yC_5lj5', '5lj5', '_yC_5lj5'],
          ['yPre_6g90', '6g90', '_yPre_6g90'],
          ['yE_6n7r', '6n7r', '_yE_6n7r'],
          ['yE_6n7p', '6n7p', '_yE_6n7p'],
         ]


strs = []
for s in strucs:
    strs.append(s[1])
strs = str(strs)    #strs = "['5gm6', '5zwo', '5lj3', '5gm6', '5mps', '6exn', '5ylz', '5y88', '3jb9', '6icz']"

#strucs = [['S3_C_5lj3', '5lj3']] # for debugging
txt = """
try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
    # sys.exit(0)

def spl_extract():
 for name in cmd.get_names("all"):
    #if name in """ + strs + """: # this should be auto
    print(" \ Extracting mode for %s" % name)
"""

for struc_col, pdb_id, complex_code in strucs:
    txt += "    if '" + pdb_id + "' in name.lower():\n"
    for index, i in df.iterrows():
        #print(i)
        ref = str(i['chain']).strip()
        struc = str(i[struc_col]).strip()
        # print(ref, struc)
        # todo fix  cmd.extract("CDC40 (PRP17, SLU4, XRS2)", "chain n")
        i['protein'] = i['protein'].split()[0]
        if struc != 'nan' and struc != '-': # if ref != 'nan' # i dont' use reference chain any more
            if ',' in struc:
                subchains = struc.split(',') # a,P,h
                for index , sc in enumerate(subchains):
                    sc = sc.strip()
                    # cmd.extract(name, selection, source_state, target_state)
                    txt += '        cmd.extract("' + i['protein'] + '_' + str(index + 1)                     + complex_code + '", "chain ' + sc + ' and ' + pdb_id + '")\n'
                # one mode to see these two proteins
                #print('        cmd.extract("' + i['protein'] + '", "chain ' + '+'.join(subchains) + '")')
            else:
                #if ref != struc:
                txt += '        cmd.extract("' + i['protein'] + complex_code + '", "chain ' + struc + ' and ' + pdb_id + '")\n'
    # the rest make unkown at the very end
    txt += '        cmd.set_name("' + pdb_id + '", "unknown_other' + complex_code + '")\n'
    txt += '        cmd.group("' + complex_code.replace('_', '') + '", "*' + complex_code + '")\n'
    txt += '        cmd.do("order *, yes")\n'
print(txt)
with open('/home/magnus/work-src/rna-tools/rna_tools/tools/PyMOL4RNA/code_for_spl.py', 'w') as f:
    f.write(txt)


# In[113]:


txt = """
try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
    # sys.exit(0)

def spl_color():
 for name in cmd.get_names("all"):
  # cmd.do('color grey50') # off gray
  if name in """ + strs + """: # this should be auto
    print(" \ Extracting mode for %s" % name)
"""

for struc_col, pdb_id, complex_code in strucs:
    txt += "    if '" + pdb_id + "' in name.lower():\n"
    for index, i in df.iterrows():
        ref = str(i['chain']).strip()
        struc = str(i[struc_col]).strip()
        # print(ref, struc)
        # todo fix  cmd.extract("CDC40 (PRP17, SLU4, XRS2)", "chain n")
        i['protein'] = i['protein'].split()[0]
        if struc != 'nan' or struc != '-': # if ref != 'nan' # i dont' use reference chain any more
            if ',' in struc:
                subchains = struc.split(',') # a,P,h
                for index , sc in enumerate(subchains):
                    sc = sc.strip()
                    # cmd.extract(name, selection, source_state, target_state)
                    txt += '        cmd.do("color ' + i['color'] +                     ', chain ' + sc + ' and ' + pdb_id + '")\n'
                # one mode to see these two proteins
                #print('        cmd.extract("' + i['protein'] + '", "chain ' + '+'.join(subchains) + '")')
            else:
                #if ref != struc:
                txt += '        #' + i['protein'] +'\n'
                try:
                    txt += '        cmd.do("color ' + i['color'] + ', chain ' + struc + ' and ' +                 pdb_id + '")' + '\n'
                except TypeError:
                    print('Color entry is missing for ' + i['protein'])
                    sys.exit(1)

                # color <object>
                txt += '        cmd.do("color ' + i['color'] + ', ' + i['protein'] +                 complex_code + '")\n'
print(txt)

with open('/home/magnus/work-src/rna-tools/rna_tools/tools/PyMOL4RNA/code_for_color_spl.py', 'w') as f:
    f.write(txt)

#strucs = [['S3_C_5lj3', '5lj3']] # for debugging
txt = """
try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
    # sys.exit(0)

def spl_color():
  #cmd.do('color grey50') # PRP8
"""

for struc_col, pdb_id, complex_code in strucs:
    txt += '  if True: # fake if, just a quick hack\n'
    for index, i in df.iterrows():
        ref = str(i['chain']).strip()
        struc = str(i[struc_col]).strip()
        # print(ref, struc)
        # todo fix  cmd.extract("CDC40 (PRP17, SLU4, XRS2)", "chain n")
        i['protein'] = i['protein'].split()[0]
        if struc != 'nan' and struc != '-': # if ref != 'nan' # i dont' use reference chain any more
            if ',' in struc:
                subchains = struc.split(',') # a,P,h
                for index , sc in enumerate(subchains):
                    sc = sc.strip()
                    # cmd.extract(name, selection, source_state, target_state)
                    txt += '        cmd.do("color ' + i['color'] + ', ' + i['protein'] + '_' + str(index + 1) +                     complex_code + '")\n'
                # one mode to see these two proteins
                #print('        cmd.extract("' + i['protein'] + '", "chain ' + '+'.join(subchains) + '")')
            else:
                #if ref != struc:
                txt += '        cmd.do("color ' + i['color'] + ', ' + i['protein'] + complex_code + '")\n'
print(txt)
with open('/home/magnus/work-src/rna-tools/rna_tools/tools/PyMOL4RNA/code_for_color_spl_objects.py', 'w') as f:
    f.write(txt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse

SECOND_PK_CHAR = ['{', '}']

def get_one_line(lines):
    if len(lines) == 2:
            ss = ''
            for i,j in zip(lines[0], lines[1]):
                if j == '(':
                    i = '['
                if j == ')':
                    i = ']'
                ss += i
    if len(lines) == 3:
            ss = ''
            for i,j,k in zip(lines[0], lines[1], lines[2]):
                if j == '(':
                    i = '['
                if j == ')':
                    i = ']'
                if k == '(':
                    i = SECOND_PK_CHAR[0]
                if k == ')':
                    i = SECOND_PK_CHAR[1]
                ss += i
    return ss

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('file', help='file')
    args = parser.parse_args()
    if args.file:
        ss = open(args.file)
        print((ss.readline().strip()))
        lines = ss.readlines()
        print((get_one_line(lines)))
        



#!/usr/bin/env python

"""Usage::

    $ rna_align_find_seq_in_alignment.py -a pistol_noPk.sto -s hcf.fa
    ('Match:', 'HCF12C_58327/229-301')
    ID: HCF12C_58327/229-301
    Name: HCF12C_58327
    Description: HCF12C_58327/229-301
    Number of features: 0
    /start=229
    /end=301
    /accession=HCF12C_58327
    Per letter annotation for: secondary_structure
    Seq('UGCCGUUUG-AGCGGCA-UUAAACA--GGUCU-UAAGCUCAA-AGCG-UCACCG...ACA', SingleLetterAlphabet())

In input file and input alignment all `-` are removed.

Or use -v::

    $ rna_align_find_seq_in_alignment.py -s 4lvv.seq -a RF01831.stockholm.sto -v
    UCAGAGUAGAAAACGAUGCGUUAAGUGUCCAGCAGACGGGGAGUUGCUGCCGGAACGAAAAGCAAAGCUUGCGGUAUCGUUUUCGCAUCCCGCUGA
    GCAGAGUAGACACAUGUGCGUUAAGUGCCGGAUGAACAGGGAGUUGUCACCCGGACGAAAAGAAAAUCUUGCGGUACAUGAGUCGCAUCCCGCUGC
    GCAGAGUAGGUUUGUGUGCGUUAAGUGCUGGUUGAACAGGGAGUUGUCAGCCGGACGAAAAGAUUUUCUUGCGGUACACGAAUCGCAUCCCGCUGC

so you can combine this with `grep`::

   $ rna_align_find_seq_in_alignment.py -s 4lvv.seq -a RF01831.stockholm.sto -v | grep UGUUGGGGUAGGAAUUACCGAGUUAUUGUCCAGCGGAACGGAAUGUGAACGCU
   UGUUGGGGUAGGAAUUACCGAGUUAUUGUCCAGCGGAACGGAAUGUGAACGCUGGAAGGAAGUAUUAGCUGCUUGAUAAUUUCGCAUUCACCACA

.. warning:: requires http://biopython.org/DIST/docs/api/Bio.AlignIO.StockholmIO-module.html

"""

from rna_tools.tools.rna_alignment.rna_alignment import RNAalignment

import sys
import argparse
from Bio import AlignIO


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-a', '--alignment', help="alignment in stockholm format", required=True)
    parser.add_argument('-s', '--seq', help="fasta seq", required=True)
    parser.add_argument('-v', '--verbose', help="", action='store_true', default=False)
    return parser

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    #args = parser.parse_args(['-f' , 'test_data/4lvv.seq', '-a', 'test_data/RF01831.stockholm.sto'])

    f = open(args.seq)
    header = f.readline().strip()
    seq = f.readline().strip()

    a = RNAalignment(args.alignment)
    a.find_seq(seq, args.verbose)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_pairs_diff.py - get a diff of pairs

Usage::

    $ rna_pairs_diff.py pistol_dca_all.bp pistol.bp
    # of ec_paris: 31
    # of ssbps   : 18
    dalta#       : 13
    [[4, 32], [6, 9], [6, 36], [6, 39], [9, 39], [13, 32], [16, 17], [17, 18], [22, 49], [29, 58]]

"""
from __future__ import print_function

import argparse


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('pairs1', help="a list of pairs, A")
    parser.add_argument('pairs2', help="a list of pairs to subtract, A-B, results in C"
                        "(all pairs that are in A and are not in B")
    parser.add_argument("-v", "--verbose", action='count',
                        help="be verbose")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    pairs1 = eval(open(args.pairs1).read().strip())
    pairs2 = eval(open(args.pairs2).read().strip())

    # go ever dca and keep if not in ss
    pairsnonss = []
    for pair in pairs1:
        if pair in pairs2:
            pass
        else:
            pairsnonss.append(pair)
    print('# of ec_paris:', len(pairs1))
    print('# of ssbps   :', len(pairs2))
    print('dalta#       :', len(pairsnonss))
    pairsnonss.sort()
    print(pairsnonss)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_pairs2SimRNArestrs.py - convert pairs to SimRNA restraints

Example::

    $ rna_paris2rfilter.py rp06_MohPairs.bp
    d A5-A42< 100 1
    d A5-A42< 100 1
    d A11-A26< 100 1
    d A11-A26< 100 1

Input::

    $ head rp06_MohPairs.bp
    [[5, 42], [11, 26], [24, 89], [26, 44], [26, 75],

"""
from __future__ import print_function
import argparse


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('pairs', help="a file with [[2, 172], [3, 169], [12, 32], [13, 31]]")
    parser.add_argument("--offset", help="can be -10", default=0, type=int)
    parser.add_argument("--weight", default=3, type=float, help="weight")
    parser.add_argument("--distance", default=10, type=float, help="distance")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    pairs = eval(open(args.pairs).read().strip())
    if args.verbose:
        print('# of pairs:', len(pairs))

    for pair in pairs:

        a, b = pair
        a += args.offset
        b += args.offset

        a = str(a)
        b = str(b)

        # hmmm.. why 2 lines here?
        print('d:A' + a + '-A' + b + ' < %.2f %.2f' % (args.distance, args.weight))
        #print('d A' + a + '-A' + b + ' < %f %f' % (args.distance, args.weight)
